@@ -2,7 +2,7 @@
 
   eval.c -
 
-  $Author: nagachika $
+  $Author$
   created at: Thu Jun 10 14:22:17 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -25,7 +25,6 @@ VALUE rb_eLocalJumpError;
 VALUE rb_eSysStackError;
 
 ID ruby_static_id_signo, ruby_static_id_status;
-static ID id_cause;
 #define id_signo ruby_static_id_signo
 #define id_status ruby_static_id_status
 
@@ -443,6 +442,9 @@ static VALUE get_thread_errinfo(rb_thread_t *th);
 static VALUE
 exc_setup_cause(VALUE exc, VALUE cause)
 {
+    ID id_cause;
+    CONST_ID(id_cause, "cause");
+
 #if SUPPORT_JOKE
     if (NIL_P(cause)) {
 	ID id_true_cause;
@@ -486,15 +488,10 @@ setup_exception(rb_thread_t *th, int tag, volatile VALUE mesg, VALUE cause)
 	mesg = rb_exc_new(rb_eRuntimeError, 0, 0);
 	nocause = 0;
     }
-    if (cause != Qundef) {
-	exc_setup_cause(mesg, cause);
-    }
-    else if (nocause) {
-	exc_setup_cause(mesg, Qnil);
-    }
-    else if (!rb_ivar_defined(mesg, id_cause)) {
-	exc_setup_cause(mesg, get_thread_errinfo(th));
+    if (cause == Qundef) {
+	cause = nocause ? Qnil : get_thread_errinfo(th);
     }
+    exc_setup_cause(mesg, cause);
 
     file = rb_source_loc(&line);
     if (file && !NIL_P(mesg)) {
@@ -509,25 +506,13 @@ setup_exception(rb_thread_t *th, int tag, volatile VALUE mesg, VALUE cause)
 		rb_ivar_set(mesg, idBt_locations, at);
 	    }
 	}
-	else {
-	    int status;
-
-	    TH_PUSH_TAG(th);
-	    if ((status = EXEC_TAG()) == 0) {
-		VALUE bt;
-		if (rb_threadptr_set_raised(th)) goto fatal;
-		bt = rb_get_backtrace(mesg);
-		if (NIL_P(bt)) {
-		    at = rb_vm_backtrace_object();
-		    if (OBJ_FROZEN(mesg)) {
-			mesg = rb_obj_dup(mesg);
-		    }
-		    rb_ivar_set(mesg, idBt_locations, at);
-		    set_backtrace(mesg, at);
-		}
-		rb_threadptr_reset_raised(th);
+	else if (NIL_P(get_backtrace(mesg))) {
+	    at = rb_vm_backtrace_object();
+	    if (OBJ_FROZEN(mesg)) {
+		mesg = rb_obj_dup(mesg);
 	    }
-	    TH_POP_TAG();
+	    rb_ivar_set(mesg, idBt_locations, at);
+	    set_backtrace(mesg, at);
 	}
     }
 
@@ -569,7 +554,6 @@ setup_exception(rb_thread_t *th, int tag, volatile VALUE mesg, VALUE cause)
     }
 
     if (rb_threadptr_set_raised(th)) {
-      fatal:
 	th->errinfo = exception_error;
 	rb_threadptr_reset_raised(th);
 	JUMP_TAG(TAG_FATAL);
@@ -1538,7 +1522,7 @@ errat_getter(ID id)
 {
     VALUE err = get_errinfo();
     if (!NIL_P(err)) {
-	return rb_get_backtrace(err);
+	return get_backtrace(err);
     }
     else {
 	return Qnil;
@@ -1668,5 +1652,4 @@ Init_eval(void)
 
     id_signo = rb_intern_const("signo");
     id_status = rb_intern_const("status");
-    id_cause = rb_intern_const("cause");
 }
