@@ -2,14 +2,14 @@
 
   digest.c -
 
-  $Author: nobu $
+  $Author$
   created at: Fri May 25 08:57:27 JST 2001
 
   Copyright (C) 1995-2001 Yukihiro Matsumoto
   Copyright (C) 2001-2006 Akinori MUSHA
 
   $RoughId: digest.c,v 1.16 2001/07/13 15:38:27 knu Exp $
-  $Id: digest.c 48662 2014-12-01 06:38:04Z nobu $
+  $Id$
 
 ************************************************/
 
@@ -124,6 +124,8 @@ hexencode_str_new(VALUE str_digest)
         p[i + i + 1] = hex[byte & 0x0f];
     }
 
+    RB_GC_GUARD(str_digest);
+
     return str;
 }
 
@@ -565,11 +567,10 @@ rb_digest_base_alloc(VALUE klass)
 
     algo = get_digest_base_metadata(klass);
 
-    pctx = xmalloc(algo->ctx_size);
+    obj = rb_data_typed_object_zalloc(klass, algo->ctx_size, &digest_type);
+    pctx = RTYPEDDATA_DATA(obj);
     algo_init(algo, pctx);
 
-    obj = TypedData_Wrap_Struct(klass, &digest_type, pctx);
-
     return obj;
 }
 
@@ -624,6 +625,7 @@ rb_digest_base_update(VALUE self, VALUE str)
 
     StringValue(str);
     algo->update_func(pctx, (unsigned char *)RSTRING_PTR(str), RSTRING_LEN(str));
+    RB_GC_GUARD(str);
 
     return self;
 }
