@@ -1,21 +1,22 @@
 /*
- * $Id: ossl_ocsp.c 48798 2014-12-12 21:58:22Z nobu $
  * 'OpenSSL for Ruby' project
  * Copyright (C) 2003  Michal Rokos <m.rokos@sh.cvut.cz>
  * Copyright (C) 2003  GOTOU Yuuzou <gotoyuzo@notwork.org>
  * All rights reserved.
  */
 /*
- * This program is licenced under the same licence as Ruby.
+ * This program is licensed under the same licence as Ruby.
  * (See the file 'LICENCE'.)
  */
 #include "ossl.h"
 
 #if defined(OSSL_OCSP_ENABLED)
 
-#define WrapOCSPReq(klass, obj, req) do { \
+#define NewOCSPReq(klass) \
+    TypedData_Wrap_Struct((klass), &ossl_ocsp_request_type, 0)
+#define SetOCSPReq(obj, req) do { \
     if(!(req)) ossl_raise(rb_eRuntimeError, "Request wasn't initialized!"); \
-    (obj) = TypedData_Wrap_Struct((klass), &ossl_ocsp_request_type, (req)); \
+    RTYPEDDATA_DATA(obj) = (req); \
 } while (0)
 #define GetOCSPReq(obj, req) do { \
     TypedData_Get_Struct((obj), OCSP_REQUEST, &ossl_ocsp_request_type, (req)); \
@@ -26,9 +27,11 @@
     GetOCSPReq((obj), (req)); \
 } while (0)
 
-#define WrapOCSPRes(klass, obj, res) do { \
+#define NewOCSPRes(klass) \
+    TypedData_Wrap_Struct((klass), &ossl_ocsp_response_type, 0)
+#define SetOCSPRes(obj, res) do { \
     if(!(res)) ossl_raise(rb_eRuntimeError, "Response wasn't initialized!"); \
-    (obj) = TypedData_Wrap_Struct((klass), &ossl_ocsp_response_type, (res)); \
+    RTYPEDDATA_DATA(obj) = (res); \
 } while (0)
 #define GetOCSPRes(obj, res) do { \
     TypedData_Get_Struct((obj), OCSP_RESPONSE, &ossl_ocsp_response_type, (res)); \
@@ -39,9 +42,11 @@
     GetOCSPRes((obj), (res)); \
 } while (0)
 
-#define WrapOCSPBasicRes(klass, obj, res) do { \
+#define NewOCSPBasicRes(klass) \
+    TypedData_Wrap_Struct((klass), &ossl_ocsp_basicresp_type, 0)
+#define SetOCSPBasicRes(obj, res) do { \
     if(!(res)) ossl_raise(rb_eRuntimeError, "Response wasn't initialized!"); \
-    (obj) = TypedData_Wrap_Struct((klass), &ossl_ocsp_basicresp_type, (res)); \
+    RTYPEDDATA_DATA(obj) = (res); \
 } while (0)
 #define GetOCSPBasicRes(obj, res) do { \
     TypedData_Get_Struct((obj), OCSP_BASICRESP, &ossl_ocsp_basicresp_type, (res)); \
@@ -52,9 +57,11 @@
     GetOCSPBasicRes((obj), (res)); \
 } while (0)
 
-#define WrapOCSPCertId(klass, obj, cid) do { \
+#define NewOCSPCertId(klass) \
+    TypedData_Wrap_Struct((klass), &ossl_ocsp_certid_type, 0)
+#define SetOCSPCertId(obj, cid) do { \
     if(!(cid)) ossl_raise(rb_eRuntimeError, "Cert ID wasn't initialized!"); \
-    (obj) = TypedData_Wrap_Struct((klass), &ossl_ocsp_certid_type, (cid)); \
+    RTYPEDDATA_DATA(obj) = (cid); \
 } while (0)
 #define GetOCSPCertId(obj, cid) do { \
     TypedData_Get_Struct((obj), OCSP_CERTID, &ossl_ocsp_certid_type, (cid)); \
@@ -134,8 +141,8 @@ static const rb_data_type_t ossl_ocsp_certid_type = {
 static VALUE
 ossl_ocspcertid_new(OCSP_CERTID *cid)
 {
-    VALUE obj;
-    WrapOCSPCertId(cOCSPCertId, obj, cid);
+    VALUE obj = NewOCSPCertId(cOCSPCertId);
+    SetOCSPCertId(obj, cid);
     return obj;
 }
 
@@ -148,9 +155,10 @@ ossl_ocspreq_alloc(VALUE klass)
     OCSP_REQUEST *req;
     VALUE obj;
 
+    obj = NewOCSPReq(klass);
     if (!(req = OCSP_REQUEST_new()))
 	ossl_raise(eOCSPError, NULL);
-    WrapOCSPReq(klass, obj, req);
+    SetOCSPReq(obj, req);
 
     return obj;
 }
@@ -294,9 +302,10 @@ ossl_ocspreq_get_certid(VALUE self)
     ary = (count > 0) ? rb_ary_new() : Qnil;
     for(i = 0; i < count; i++){
 	one = OCSP_request_onereq_get0(req, i);
+	tmp = NewOCSPCertId(cOCSPCertId);
 	if(!(id = OCSP_CERTID_dup(OCSP_onereq_get0_id(one))))
 	    ossl_raise(eOCSPError, NULL);
-	WrapOCSPCertId(cOCSPCertId, tmp, id);
+	SetOCSPCertId(tmp, id);
 	rb_ary_push(ary, tmp);
     }
 
@@ -415,9 +424,10 @@ ossl_ocspres_s_create(VALUE klass, VALUE status, VALUE basic_resp)
 
     if(NIL_P(basic_resp)) bs = NULL;
     else GetOCSPBasicRes(basic_resp, bs); /* NO NEED TO DUP */
+    obj = NewOCSPRes(klass);
     if(!(res = OCSP_response_create(st, bs)))
 	ossl_raise(eOCSPError, NULL);
-    WrapOCSPRes(klass, obj, res);
+    SetOCSPRes(obj, res);
 
     return obj;
 }
@@ -428,9 +438,10 @@ ossl_ocspres_alloc(VALUE klass)
     OCSP_RESPONSE *res;
     VALUE obj;
 
+    obj = NewOCSPRes(klass);
     if(!(res = OCSP_RESPONSE_new()))
 	ossl_raise(eOCSPError, NULL);
-    WrapOCSPRes(klass, obj, res);
+    SetOCSPRes(obj, res);
 
     return obj;
 }
@@ -519,9 +530,10 @@ ossl_ocspres_get_basic(VALUE self)
     VALUE ret;
 
     GetOCSPRes(self, res);
+    ret = NewOCSPBasicRes(cOCSPBasicRes);
     if(!(bs = OCSP_response_get1_basic(res)))
 	return Qnil;
-    WrapOCSPBasicRes(cOCSPBasicRes, ret, bs);
+    SetOCSPBasicRes(ret, bs);
 
     return ret;
 }
@@ -562,9 +574,10 @@ ossl_ocspbres_alloc(VALUE klass)
     OCSP_BASICRESP *bs;
     VALUE obj;
 
+    obj = NewOCSPBasicRes(klass);
     if(!(bs = OCSP_BASICRESP_new()))
 	ossl_raise(eOCSPError, NULL);
-    WrapOCSPBasicRes(klass, obj, bs);
+    SetOCSPBasicRes(obj, bs);
 
     return obj;
 }
@@ -638,7 +651,7 @@ ossl_ocspbres_add_nonce(int argc, VALUE *argv, VALUE self)
  * call-seq:
  *   basic_response.add_status(certificate_id, status, reason, revocation_time, this_update, next_update, extensions) -> basic_response
  *
- * Adds a validation +status+ (0 for revoked, 1 for success) to this
+ * Adds a validation +status+ (0 for good, 1 for revoked, 2 for unknown) to this
  * response for +certificate_id+.  +reason+ describes the reason for the
  * revocation, if any.
  *
@@ -658,9 +671,9 @@ ossl_ocspbres_add_status(VALUE self, VALUE cid, VALUE status,
     OCSP_BASICRESP *bs;
     OCSP_SINGLERESP *single;
     OCSP_CERTID *id;
-    int st, rsn;
     ASN1_TIME *ths, *nxt, *rev;
-    int error, i, rstatus = 0;
+    int st, rsn, error, rstatus = 0;
+    long i;
     VALUE tmp;
 
     st = NUM2INT(status);
@@ -669,7 +682,7 @@ ossl_ocspbres_add_status(VALUE self, VALUE cid, VALUE status,
 	/* All ary's members should be X509Extension */
 	Check_Type(ext, T_ARRAY);
 	for (i = 0; i < RARRAY_LEN(ext); i++)
-	    OSSL_Check_Kind(RARRAY_PTR(ext)[i], cX509Ext);
+	    OSSL_Check_Kind(RARRAY_AREF(ext, i), cX509Ext);
     }
 
     error = 0;
@@ -698,7 +711,7 @@ ossl_ocspbres_add_status(VALUE self, VALUE cid, VALUE status,
 	sk_X509_EXTENSION_pop_free(single->singleExtensions, X509_EXTENSION_free);
 	single->singleExtensions = NULL;
 	for(i = 0; i < RARRAY_LEN(ext); i++){
-	    x509ext = DupX509ExtPtr(RARRAY_PTR(ext)[i]);
+	    x509ext = DupX509ExtPtr(RARRAY_AREF(ext, i));
 	    if(!OCSP_SINGLERESP_add_ext(single, x509ext, -1)){
 		X509_EXTENSION_free(x509ext);
 		error = 1;
@@ -723,7 +736,7 @@ ossl_ocspbres_add_status(VALUE self, VALUE cid, VALUE status,
  *   basic_response.status -> statuses
  *
  * Returns an Array of statuses for this response.  Each status contains a
- * CertificateId, the status (0 for success, 1 for revoked), the reason for
+ * CertificateId, the status (0 for good, 1 for revoked, 2 for unknown), the reason for
  * the status, the revocation time, the time of this update, the time for the
  * next update and a list of OpenSSL::X509::Extensions.
  */
@@ -851,9 +864,10 @@ ossl_ocspcid_alloc(VALUE klass)
     OCSP_CERTID *id;
     VALUE obj;
 
+    obj = NewOCSPCertId(klass);
     if(!(id = OCSP_CERTID_new()))
 	ossl_raise(eOCSPError, NULL);
-    WrapOCSPCertId(klass, obj, id);
+    SetOCSPCertId(obj, id);
 
     return obj;
 }
