@@ -1,11 +1,10 @@
 /*
- * $Id: ossl_ssl.h 48801 2014-12-12 21:58:34Z nobu $
  * 'OpenSSL for Ruby' project
  * Copyright (C) 2001-2002  Michal Rokos <m.rokos@sh.cvut.cz>
  * All rights reserved.
  */
 /*
- * This program is licenced under the same licence as Ruby.
+ * This program is licensed under the same licence as Ruby.
  * (See the file 'LICENCE'.)
  */
 #if !defined(_OSSL_SSL_H_)
@@ -30,13 +29,10 @@
 extern const rb_data_type_t ossl_ssl_type;
 extern const rb_data_type_t ossl_ssl_session_type;
 extern VALUE mSSL;
-extern VALUE eSSLError;
 extern VALUE cSSLSocket;
-extern VALUE cSSLContext;
 extern VALUE cSSLSession;
 
 void Init_ossl_ssl(void);
 void Init_ossl_ssl_session(void);
 
 #endif /* _OSSL_SSL_H_ */
-
