@@ -1,4 +1,5 @@
 #!ruby -s
+# frozen_string_literal: false
 
 require 'mkmf'
 require 'rbconfig'
@@ -6,8 +7,7 @@ require 'rbconfig'
 def main
   unless find_executable('bison')
     unless File.exist?('ripper.c') or File.exist?("#{$srcdir}/ripper.c")
-      Logging.message 'missing bison; abort'
-      return
+      raise 'missing bison; abort'
     end
   end
   $objs = %w(ripper.o)
