@@ -301,16 +301,11 @@ static VALUE
 sockopt_bool(VALUE self)
 {
     int i;
-    long len;
     VALUE data = sockopt_data(self);
     StringValue(data);
-    len = RSTRING_LEN(data);
-    if (len == 1) {
-        return *RSTRING_PTR(data) == 0 ? Qfalse : Qtrue;
-    }
-    if (len != sizeof(int))
+    if (RSTRING_LEN(data) != sizeof(int))
         rb_raise(rb_eTypeError, "size differ.  expected as sizeof(int)=%d but %ld",
-                 (int)sizeof(int), (long)len);
+                 (int)sizeof(int), (long)RSTRING_LEN(data));
     memcpy((char*)&i, RSTRING_PTR(data), sizeof(int));
     return i == 0 ? Qfalse : Qtrue;
 }
@@ -648,7 +643,7 @@ inspect_timeval_as_interval(int level, int optname, VALUE data, VALUE ret)
  */
 
 #if !defined HAVE_INET_NTOP && ! defined _WIN32
-const char *
+static const char *
 inet_ntop(int af, const void *addr, char *numaddr, size_t numaddr_len)
 {
 #ifdef HAVE_INET_NTOA
@@ -663,6 +658,10 @@ inet_ntop(int af, const void *addr, char *numaddr, size_t numaddr_len)
 #endif
     return numaddr;
 }
+#elif defined __MINGW32__
+# define inet_ntop(f,a,n,l)      rb_w32_inet_ntop(f,a,n,l)
+#elif defined _MSC_VER && RUBY_MSVCRT_VERSION < 90
+const char *WSAAPI inet_ntop(int, const void *, char *, size_t);
 #endif
 
 /* Although the buffer size needed depends on the prefixes, "%u" may generate "4294967295".  */
@@ -929,12 +928,7 @@ inspect_tcpi_usec(VALUE ret, const char *prefix, uint32_t t)
     rb_str_catf(ret, "%s%u.%06us", prefix, t / 1000000, t % 1000000);
 }
 
-#if !defined __FreeBSD__ && ( \
-    defined HAVE_STRUCT_TCP_INFO_TCPI_LAST_DATA_SENT || \
-    defined HAVE_STRUCT_TCP_INFO_TCPI_LAST_DATA_RECV || \
-    defined HAVE_STRUCT_TCP_INFO_TCPI_LAST_ACK_SENT  || \
-    defined HAVE_STRUCT_TCP_INFO_TCPI_LAST_ACK_RECV  || \
-    0)
+#if defined(__linux__) || defined(__sun)
 static void
 inspect_tcpi_msec(VALUE ret, const char *prefix, uint32_t t)
 {
@@ -1250,7 +1244,7 @@ sockopt_inspect(VALUE self)
 
 	v = optname_to_sym(level, optname);
 	if (SYMBOL_P(v))
-	    rb_str_catf(ret, " %s", rb_id2name(SYM2ID(v)));
+	    rb_str_catf(ret, " %"PRIsVALUE, rb_sym2str(v));
 	else
 	    rb_str_catf(ret, " optname:%d", optname);
     }
