@@ -25,28 +25,6 @@ unixsock_connect_internal(VALUE a)
 			        arg->sockaddrlen, 0);
 }
 
-static VALUE
-unixsock_path_value(VALUE path)
-{
-#ifdef __linux__
-#define TO_STR_FOR_LINUX_ABSTRACT_NAMESPACE 0
-
-    VALUE name = path;
-#if TO_STR_FOR_LINUX_ABSTRACT_NAMESPACE
-    const int isstr = !NIL_P(name = rb_check_string_type(name));
-#else
-    const int isstr = RB_TYPE_P(name, T_STRING);
-#endif
-    if (isstr) {
-        if (RSTRING_LEN(name) == 0 || RSTRING_PTR(name)[0] == '\0') {
-            rb_check_safe_obj(name);
-            return name;             /* ignore encoding */
-        }
-    }
-#endif
-    return rb_get_path(path);
-}
-
 VALUE
 rsock_init_unixsock(VALUE sock, VALUE path, int server)
 {
@@ -55,7 +33,7 @@ rsock_init_unixsock(VALUE sock, VALUE path, int server)
     int fd, status;
     rb_io_t *fptr;
 
-    path = unixsock_path_value(path);
+    SafeStringValue(path);
 
     INIT_SOCKADDR_UN(&sockaddr, sizeof(struct sockaddr_un));
     if (sizeof(sockaddr.sun_path) < (size_t)RSTRING_LEN(path)) {
@@ -87,14 +65,16 @@ rsock_init_unixsock(VALUE sock, VALUE path, int server)
     }
 
     if (status < 0) {
+	int e = errno;
 	close(fd);
-        rsock_sys_fail_path("connect(2)", path);
+	rsock_syserr_fail_path(e, "connect(2)", path);
     }
 
     if (server) {
 	if (listen(fd, SOMAXCONN) < 0) {
+	    int e = errno;
 	    close(fd);
-            rsock_sys_fail_path("listen(2)", path);
+	    rsock_syserr_fail_path(e, "listen(2)", path);
 	}
     }
 
@@ -153,7 +133,7 @@ unix_path(VALUE sock)
 
 /*
  * call-seq:
- *   unixsocket.recvfrom(maxlen [, flags]) => [mesg, unixaddress]
+ *   unixsocket.recvfrom(maxlen [, flags[, outbuf]]) => [mesg, unixaddress]
  *
  * Receives a message via _unixsocket_.
  *
@@ -161,6 +141,9 @@ unix_path(VALUE sock)
  *
  * _flags_ should be a bitwise OR of Socket::MSG_* constants.
  *
+ * _outbuf_ will contain only the received data after the method call
+ * even if it is not empty at the beginning.
+ *
  *   s1 = Socket.new(:UNIX, :DGRAM, 0)
  *   s1_ai = Addrinfo.unix("/tmp/sock1")
  *   s1.bind(s1_ai)
@@ -220,6 +203,8 @@ sendmsg_blocking(void *data)
  *   p stdout.fileno #=> 6
  *
  *   stdout.puts "hello" # outputs "hello\n" to standard output.
+ *
+ * _io_ may be any kind of IO object or integer file descriptor.
  */
 static VALUE
 unix_send_io(VALUE sock, VALUE val)
@@ -316,6 +301,11 @@ recvmsg_blocking(void *data)
  *     }
  *   }
  *
+ * _klass_ will determine the class of _io_ returned (using the
+ * IO.for_fd singleton method or similar).
+ * If _klass_ is +nil+, an integer file descriptor is returned.
+ *
+ * _mode_ is the same as the argument passed to IO.for_fd
  */
 static VALUE
 unix_recv_io(int argc, VALUE *argv, VALUE sock)
