@@ -1,3 +1,4 @@
+# frozen_string_literal: false
 require 'win32/importer'
 
 module Win32
@@ -332,11 +333,11 @@ For detail, see the MSDN[http://msdn.microsoft.com/library/en-us/sysinfo/base/pr
       end
 
       def DeleteValue(hkey, name)
-        check RegDeleteValueW.call(hkey, make_wstr(name))
+        check RegDeleteValue.call(hkey, make_wstr(name))
       end
 
       def DeleteKey(hkey, name)
-        check RegDeleteKeyW.call(hkey, make_wstr(name))
+        check RegDeleteKey.call(hkey, make_wstr(name))
       end
 
       def FlushKey(hkey)
@@ -377,15 +378,16 @@ For detail, see the MSDN[http://msdn.microsoft.com/library/en-us/sysinfo/base/pr
       }
     end
 
-    @@type2name = { }
-    %w[
+    @@type2name = %w[
       REG_NONE REG_SZ REG_EXPAND_SZ REG_BINARY REG_DWORD
       REG_DWORD_BIG_ENDIAN REG_LINK REG_MULTI_SZ
       REG_RESOURCE_LIST REG_FULL_RESOURCE_DESCRIPTOR
       REG_RESOURCE_REQUIREMENTS_LIST REG_QWORD
-    ].each do |type|
-      @@type2name[Constants.const_get(type)] = type
-    end
+    ].inject([]) do |ary, type|
+      type.freeze
+      ary[Constants.const_get(type)] = type
+      ary
+    end.freeze
 
     #
     # Convert registry type value to readable string.
@@ -641,9 +643,7 @@ For detail, see the MSDN[http://msdn.microsoft.com/library/en-us/sysinfo/base/pr
     def read(name, *rtype)
       type, data = API.QueryValue(@hkey, name)
       unless rtype.empty? or rtype.include?(type)
-        raise TypeError, "Type mismatch (expect [#{
-          rtype.map{|t|Registry.type2name(t)}.join(', ')}] but #{
-          Registry.type2name(type)} present)"
+        raise TypeError, "Type mismatch (expect #{rtype.inspect} but #{type} present)"
       end
       case type
       when REG_SZ, REG_EXPAND_SZ
@@ -659,7 +659,7 @@ For detail, see the MSDN[http://msdn.microsoft.com/library/en-us/sysinfo/base/pr
       when REG_QWORD
         [ type, API.unpackqw(data) ]
       else
-        raise TypeError, "Type #{Registry.type2name(type)} is not supported."
+        raise TypeError, "Type #{type} is not supported."
       end
     end
 
@@ -682,7 +682,7 @@ For detail, see the MSDN[http://msdn.microsoft.com/library/en-us/sysinfo/base/pr
       when REG_EXPAND_SZ
         Registry.expand_environ(data)
       else
-        raise TypeError, "Type #{Registry.type2name(type)} is not supported."
+        raise TypeError, "Type #{type} is not supported."
       end
     end
 
@@ -755,7 +755,7 @@ For detail, see the MSDN[http://msdn.microsoft.com/library/en-us/sysinfo/base/pr
       when REG_QWORD
         data = API.packqw(data.to_i)
       else
-        raise TypeError, "Unsupported type #{Registry.type2name(type)}"
+        raise TypeError, "Unsupported type #{type}"
       end
       API.SetValue(@hkey, name, type, data, data.bytesize + termsize)
     end
