@@ -1,15 +1,17 @@
 # -*- coding: utf-8 -*-
-#   irb/lc/ja/error.rb - 
-#   	$Release Version: 0.9.5$
-#   	$Revision: 20882 $
+# frozen_string_literal: false
+#   irb/lc/ja/error.rb -
+#   	$Release Version: 0.9.6$
+#   	$Revision$
 #   	by Keiju ISHITSUKA(keiju@ruby-lang.org)
 #
 # --
 #
-#   
+#
 #
 require "e2mmap"
 
+# :stopdoc:
 module IRB
   # exceptions
   extend Exception2MessageMapper
@@ -23,5 +25,7 @@ module IRB
   def_exception :CantShiftToMultiIrbMode, 'multi-irb modeに移れません.'
   def_exception :CantChangeBinding, 'バインディング(%s)に変更できません.'
   def_exception :UndefinedPromptMode, 'プロンプトモード(%s)は定義されていません.'
+  def_exception :IllegalRCNameGenerator, 'RC_NAME_GENERATORが正しく定義されていません.'
 end
+# :startdoc:
 # vim:fileencoding=utf-8
