@@ -2,7 +2,7 @@
 
   object.c -
 
-  $Author: shugo $
+  $Author$
   created at: Thu Jul 15 12:01:24 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
