@@ -2,7 +2,7 @@
 
   random.c -
 
-  $Author: nobu $
+  $Author$
   created at: Fri Dec 24 16:39:21 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -77,14 +77,22 @@ The original copyright notice follows.
 #include <sys/time.h>
 #endif
 
+#ifdef HAVE_SYSCALL_H
+#include <syscall.h>
+#elif defined HAVE_SYS_SYSCALL_H
+#include <sys/syscall.h>
+#endif
+
 #ifdef _WIN32
 # if !defined(_WIN32_WINNT) || _WIN32_WINNT < 0x0400
 #  undef _WIN32_WINNT
 #  define _WIN32_WINNT 0x400
 #  undef __WINCRYPT_H__
 # endif
+#include <windows.h>
 #include <wincrypt.h>
 #endif
+#include "ruby_atomic.h"
 
 typedef int int_must_be_32bit_at_least[sizeof(int) * CHAR_BIT < 32 ? -1 : 1];
 
@@ -195,16 +203,26 @@ genrand_int32(struct MT *mt)
 }
 
 /* generates a random number on [0,1) with 53-bit resolution*/
+static double int_pair_to_real_exclusive(uint32_t a, uint32_t b);
 static double
 genrand_real(struct MT *mt)
 {
     /* mt must be initialized */
-    unsigned int a = genrand_int32(mt)>>5, b = genrand_int32(mt)>>6;
+    unsigned int a = genrand_int32(mt), b = genrand_int32(mt);
+    return int_pair_to_real_exclusive(a, b);
+}
+
+static double
+int_pair_to_real_exclusive(uint32_t a, uint32_t b)
+{
+    a >>= 5;
+    b >>= 6;
     return(a*67108864.0+b)*(1.0/9007199254740992.0);
 }
 
 /* generates a random number on [0,1] with 53-bit resolution*/
 static double int_pair_to_real_inclusive(uint32_t a, uint32_t b);
+#if 0
 static double
 genrand_real2(struct MT *mt)
 {
@@ -212,6 +230,7 @@ genrand_real2(struct MT *mt)
     uint32_t a = genrand_int32(mt), b = genrand_int32(mt);
     return int_pair_to_real_inclusive(a, b);
 }
+#endif
 
 /* These real versions are due to Isaku Wada, 2002/01/09 added */
 
@@ -324,7 +343,7 @@ random_free(void *ptr)
 static size_t
 random_memsize(const void *ptr)
 {
-    return ptr ? sizeof(rb_random_t) : 0;
+    return sizeof(rb_random_t);
 }
 
 static const rb_data_type_t random_data_type = {
@@ -433,43 +452,122 @@ random_init(int argc, VALUE *argv, VALUE obj)
 # define USE_DEV_URANDOM 0
 #endif
 
+#if USE_DEV_URANDOM
+static int
+fill_random_bytes_urandom(void *seed, size_t size)
+{
+    /*
+      O_NONBLOCK and O_NOCTTY is meaningless if /dev/urandom correctly points
+      to a urandom device. But it protects from several strange hazard if
+      /dev/urandom is not a urandom device.
+    */
+    int fd = rb_cloexec_open("/dev/urandom",
+# ifdef O_NONBLOCK
+			     O_NONBLOCK|
+# endif
+# ifdef O_NOCTTY
+			     O_NOCTTY|
+# endif
+			     O_RDONLY, 0);
+    struct stat statbuf;
+    ssize_t ret = 0;
+
+    if (fd < 0) return -1;
+    rb_update_max_fd(fd);
+    if (fstat(fd, &statbuf) == 0 && S_ISCHR(statbuf.st_mode)) {
+	ret = read(fd, seed, size);
+    }
+    close(fd);
+    if (ret < 0 || (size_t)ret < size) return -1;
+    return 0;
+}
+#else
+# define fill_random_bytes_urandom(seed, size) -1
+#endif
+
+#if defined(_WIN32)
+static void
+release_crypt(void *p)
+{
+    HCRYPTPROV prov = (HCRYPTPROV)ATOMIC_PTR_EXCHANGE(*(HCRYPTPROV *)p, INVALID_HANDLE_VALUE);
+    if (prov && prov != (HCRYPTPROV)INVALID_HANDLE_VALUE) {
+	CryptReleaseContext(prov, 0);
+    }
+}
+
+static int
+fill_random_bytes_syscall(void *seed, size_t size, int unused)
+{
+    static HCRYPTPROV perm_prov;
+    HCRYPTPROV prov = perm_prov, old_prov;
+    if (!prov) {
+	if (!CryptAcquireContext(&prov, NULL, NULL, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT)) {
+	    prov = (HCRYPTPROV)INVALID_HANDLE_VALUE;
+	}
+	old_prov = (HCRYPTPROV)ATOMIC_PTR_CAS(perm_prov, 0, prov);
+	if (LIKELY(!old_prov)) { /* no other threads acquried */
+	    if (prov != (HCRYPTPROV)INVALID_HANDLE_VALUE) {
+		rb_gc_register_mark_object(Data_Wrap_Struct(0, 0, release_crypt, &perm_prov));
+	    }
+	}
+	else {			/* another thread acquried */
+	    if (prov != (HCRYPTPROV)INVALID_HANDLE_VALUE) {
+		CryptReleaseContext(prov, 0);
+	    }
+	    prov = old_prov;
+	}
+    }
+    if (prov == (HCRYPTPROV)INVALID_HANDLE_VALUE) return -1;
+    CryptGenRandom(prov, size, seed);
+    return 0;
+}
+#elif defined __linux__ && defined SYS_getrandom
+#include <linux/random.h>
+
+# ifndef GRND_NONBLOCK
+#   define GRND_NONBLOCK 0x0001	/* not defined in musl libc */
+# endif
+
+static int
+fill_random_bytes_syscall(void *seed, size_t size, int need_secure)
+{
+    static rb_atomic_t try_syscall = 1;
+    if (try_syscall) {
+	long ret;
+	int flags = 0;
+	if (!need_secure)
+	    flags = GRND_NONBLOCK;
+	errno = 0;
+	ret = syscall(SYS_getrandom, seed, size, flags);
+	if (errno == ENOSYS) {
+	    ATOMIC_SET(try_syscall, 0);
+	    return -1;
+	}
+	if ((size_t)ret == size) return 0;
+    }
+    return -1;
+}
+#else
+# define fill_random_bytes_syscall(seed, size, need_secure) -1
+#endif
+
+static int
+fill_random_bytes(void *seed, size_t size, int need_secure)
+{
+    int ret = fill_random_bytes_syscall(seed, size, need_secure);
+    if (ret == 0) return ret;
+    return fill_random_bytes_urandom(seed, size);
+}
+
 static void
 fill_random_seed(uint32_t seed[DEFAULT_SEED_CNT])
 {
     static int n = 0;
     struct timeval tv;
-#if USE_DEV_URANDOM
-    int fd;
-    struct stat statbuf;
-#elif defined(_WIN32)
-    HCRYPTPROV prov;
-#endif
 
     memset(seed, 0, DEFAULT_SEED_LEN);
 
-#if USE_DEV_URANDOM
-    if ((fd = rb_cloexec_open("/dev/urandom", O_RDONLY
-#ifdef O_NONBLOCK
-            |O_NONBLOCK
-#endif
-#ifdef O_NOCTTY
-            |O_NOCTTY
-#endif
-            , 0)) >= 0) {
-        rb_update_max_fd(fd);
-        if (fstat(fd, &statbuf) == 0 && S_ISCHR(statbuf.st_mode)) {
-	    if (read(fd, seed, DEFAULT_SEED_LEN) < DEFAULT_SEED_LEN) {
-		/* abandon */;
-	    }
-        }
-        close(fd);
-    }
-#elif defined(_WIN32)
-    if (CryptAcquireContext(&prov, NULL, NULL, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT)) {
-	CryptGenRandom(prov, DEFAULT_SEED_LEN, (void *)seed);
-	CryptReleaseContext(prov, 0);
-    }
-#endif
+    fill_random_bytes(seed, DEFAULT_SEED_LEN, TRUE);
 
     gettimeofday(&tv, 0);
     seed[0] ^= tv.tv_usec;
@@ -524,6 +622,23 @@ random_seed(void)
     return make_seed_value(buf);
 }
 
+/*
+ * call-seq: Random.raw_seed(size) -> string
+ *
+ * Returns a raw seed string, using platform providing features.
+ *
+ *   Random.raw_seed(8)  #=> "\x78\x41\xBA\xAF\x7D\xEA\xD8\xEA"
+ */
+static VALUE
+random_raw_seed(VALUE self, VALUE size)
+{
+    long n = NUM2ULONG(size);
+    VALUE buf = rb_str_new(0, n);
+    if (n == 0) return buf;
+    if (fill_random_bytes(RSTRING_PTR(buf), n, FALSE)) return Qnil;
+    return buf;
+}
+
 /*
  * call-seq: prng.seed -> integer
  *
@@ -790,10 +905,9 @@ rb_genrand_ulong_limited(unsigned long limit)
     return limited_rand(default_mt(), limit);
 }
 
-unsigned int
-rb_random_int32(VALUE obj)
+static unsigned int
+random_int32(VALUE obj, rb_random_t *rnd)
 {
-    rb_random_t *rnd = try_get_rnd(obj);
     if (!rnd) {
 #if SIZEOF_LONG * CHAR_BIT > 32
 	VALUE lim = ULONG2NUM(0x100000000UL);
@@ -807,6 +921,25 @@ rb_random_int32(VALUE obj)
     return genrand_int32(&rnd->mt);
 }
 
+unsigned int
+rb_random_int32(VALUE obj)
+{
+    return random_int32(obj, try_get_rnd(obj));
+}
+
+static double
+random_real(VALUE obj, rb_random_t *rnd, int excl)
+{
+    uint32_t a = random_int32(obj, rnd);
+    uint32_t b = random_int32(obj, rnd);
+    if (excl) {
+	return int_pair_to_real_exclusive(a, b);
+    }
+    else {
+	return int_pair_to_real_inclusive(a, b);
+    }
+}
+
 double
 rb_random_real(VALUE obj)
 {
@@ -838,10 +971,9 @@ ulong_to_num_plus_1(unsigned long n)
 #endif
 }
 
-unsigned long
-rb_random_ulong_limited(VALUE obj, unsigned long limit)
+static unsigned long
+random_ulong_limited(VALUE obj, rb_random_t *rnd, unsigned long limit)
 {
-    rb_random_t *rnd = try_get_rnd(obj);
     if (!rnd) {
 	extern int rb_num_negative_p(VALUE);
 	VALUE lim = ulong_to_num_plus_1(limit);
@@ -858,6 +990,32 @@ rb_random_ulong_limited(VALUE obj, unsigned long limit)
     return limited_rand(&rnd->mt, limit);
 }
 
+static VALUE
+random_ulong_limited_big(VALUE obj, rb_random_t *rnd, VALUE vmax)
+{
+    if (!rnd) {
+	extern int rb_num_negative_p(VALUE);
+	VALUE lim = rb_big_plus(vmax, INT2FIX(1));
+	VALUE v = rb_to_int(rb_funcall2(obj, id_rand, 1, &lim));
+	if (rb_num_negative_p(v)) {
+	    rb_raise(rb_eRangeError, "random number too small %"PRIsVALUE, v);
+	}
+	if (FIX2LONG(rb_big_cmp(vmax, v)) < 0) {
+	    rb_raise(rb_eRangeError, "random number too big %"PRIsVALUE, v);
+	}
+	return v;
+    }
+    return limited_big_rand(&rnd->mt, vmax);
+}
+
+unsigned long
+rb_random_ulong_limited(VALUE obj, unsigned long limit)
+{
+    return random_ulong_limited(obj, try_get_rnd(obj), limit);
+}
+
+static VALUE genrand_bytes(rb_random_t *rnd, long n);
+
 /*
  * call-seq: prng.bytes(size) -> a_string
  *
@@ -869,21 +1027,16 @@ rb_random_ulong_limited(VALUE obj, unsigned long limit)
 static VALUE
 random_bytes(VALUE obj, VALUE len)
 {
-    return rb_random_bytes(obj, NUM2LONG(rb_to_int(len)));
+    return genrand_bytes(get_rnd(obj), NUM2LONG(rb_to_int(len)));
 }
 
-VALUE
-rb_random_bytes(VALUE obj, long n)
+static VALUE
+genrand_bytes(rb_random_t *rnd, long n)
 {
-    rb_random_t *rnd = try_get_rnd(obj);
     VALUE bytes;
     char *ptr;
     unsigned int r, i;
 
-    if (!rnd) {
-	VALUE len = LONG2NUM(n);
-	return rb_funcall2(obj, id_bytes, 1, &len);
-    }
     bytes = rb_str_new(0, n);
     ptr = RSTRING_PTR(bytes);
     for (; n >= SIZEOF_INT32; n -= SIZEOF_INT32) {
@@ -904,6 +1057,17 @@ rb_random_bytes(VALUE obj, long n)
     return bytes;
 }
 
+VALUE
+rb_random_bytes(VALUE obj, long n)
+{
+    rb_random_t *rnd = try_get_rnd(obj);
+    if (!rnd) {
+	VALUE len = LONG2NUM(n);
+	return rb_funcall2(obj, id_bytes, 1, &len);
+    }
+    return genrand_bytes(rnd, n);
+}
+
 static VALUE
 range_values(VALUE vmax, VALUE *begp, VALUE *endp, int *exclp)
 {
@@ -918,20 +1082,19 @@ range_values(VALUE vmax, VALUE *begp, VALUE *endp, int *exclp)
 }
 
 static VALUE
-rand_int(struct MT *mt, VALUE vmax, int restrictive)
+rand_int(VALUE obj, rb_random_t *rnd, VALUE vmax, int restrictive)
 {
     /* mt must be initialized */
-    long max;
     unsigned long r;
 
     if (FIXNUM_P(vmax)) {
-	max = FIX2LONG(vmax);
+	long max = FIX2LONG(vmax);
 	if (!max) return Qnil;
 	if (max < 0) {
 	    if (restrictive) return Qnil;
 	    max = -max;
 	}
-	r = limited_rand(mt, (unsigned long)max - 1);
+	r = random_ulong_limited(obj, rnd, (unsigned long)max - 1);
 	return ULONG2NUM(r);
     }
     else {
@@ -943,30 +1106,57 @@ rand_int(struct MT *mt, VALUE vmax, int restrictive)
 	}
 	vmax = rb_big_minus(vmax, INT2FIX(1));
 	if (FIXNUM_P(vmax)) {
-	    max = FIX2LONG(vmax);
+	    long max = FIX2LONG(vmax);
 	    if (max == -1) return Qnil;
-	    r = limited_rand(mt, max);
+	    r = random_ulong_limited(obj, rnd, max);
 	    return LONG2NUM(r);
 	}
-	ret = limited_big_rand(mt, vmax);
+	ret = random_ulong_limited_big(obj, rnd, vmax);
 	RB_GC_GUARD(vmax);
 	return ret;
     }
 }
 
+NORETURN(static void domain_error(void));
+static void
+domain_error(void)
+{
+    VALUE error = INT2FIX(EDOM);
+    rb_exc_raise(rb_class_new_instance(1, &error, rb_eSystemCallError));
+}
+
+NORETURN(static void invalid_argument(VALUE));
+static void
+invalid_argument(VALUE arg0)
+{
+    rb_raise(rb_eArgError, "invalid argument - %"PRIsVALUE, arg0);
+}
+
+static VALUE
+check_random_number(VALUE v, const VALUE *argv)
+{
+    switch (v) {
+      case Qfalse:
+	(void)NUM2LONG(argv[0]);
+	break;
+      case Qnil:
+	invalid_argument(argv[0]);
+    }
+    return v;
+}
+
 static inline double
 float_value(VALUE v)
 {
     double x = RFLOAT_VALUE(v);
     if (isinf(x) || isnan(x)) {
-	VALUE error = INT2FIX(EDOM);
-	rb_exc_raise(rb_class_new_instance(1, &error, rb_eSystemCallError));
+	domain_error();
     }
     return x;
 }
 
 static inline VALUE
-rand_range(struct MT* mt, VALUE range)
+rand_range(VALUE obj, rb_random_t* rnd, VALUE range)
 {
     VALUE beg = Qundef, end = Qundef, vmax, v;
     int excl = 0;
@@ -980,7 +1170,7 @@ rand_range(struct MT* mt, VALUE range)
 	if (FIXNUM_P(vmax)) {
 	  fixnum:
 	    if ((max = FIX2LONG(vmax) - excl) >= 0) {
-		unsigned long r = limited_rand(mt, (unsigned long)max);
+		unsigned long r = random_ulong_limited(obj, rnd, (unsigned long)max);
 		v = ULONG2NUM(r);
 	    }
 	}
@@ -990,7 +1180,7 @@ rand_range(struct MT* mt, VALUE range)
 		excl = 0;
 		goto fixnum;
 	    }
-	    v = limited_big_rand(mt, vmax);
+	    v = random_ulong_limited_big(obj, rnd, vmax);
 	}
     }
     else if (v = rb_check_to_float(vmax), !NIL_P(v)) {
@@ -1003,17 +1193,12 @@ rand_range(struct MT* mt, VALUE range)
 	    mid = max + min;
 	    max -= min;
 	}
-	else {
-	    float_value(v);
+	else if (isnan(max)) {
+	    domain_error();
 	}
 	v = Qnil;
 	if (max > 0.0) {
-	    if (excl) {
-		r = genrand_real(mt);
-	    }
-	    else {
-		r = genrand_real2(mt);
-	    }
+	    r = random_real(obj, rnd, excl);
 	    if (scale > 1) {
 		return rb_float_new(+(+(+(r - 0.5) * max) * scale) + mid);
 	    }
@@ -1046,7 +1231,7 @@ rand_range(struct MT* mt, VALUE range)
     return v;
 }
 
-static VALUE rand_random(int argc, VALUE *argv, rb_random_t *rnd);
+static VALUE rand_random(int argc, VALUE *argv, VALUE obj, rb_random_t *rnd);
 
 /*
  * call-seq:
@@ -1080,16 +1265,19 @@ static VALUE rand_random(int argc, VALUE *argv, rb_random_t *rnd);
 static VALUE
 random_rand(int argc, VALUE *argv, VALUE obj)
 {
-    return rand_random(argc, argv, get_rnd(obj));
+    VALUE v = rand_random(argc, argv, obj, get_rnd(obj));
+    check_random_number(v, argv);
+    return v;
 }
 
 static VALUE
-rand_random(int argc, VALUE *argv, rb_random_t *rnd)
+rand_random(int argc, VALUE *argv, VALUE obj, rb_random_t *rnd)
 {
     VALUE vmax, v;
+    double max = 0.0;
 
     if (argc == 0) {
-	return rb_float_new(genrand_real(&rnd->mt));
+	goto float_rand;
     }
     else {
 	rb_check_arity(argc, 0, 1);
@@ -1099,28 +1287,41 @@ rand_random(int argc, VALUE *argv, rb_random_t *rnd)
 	v = Qnil;
     }
     else if (!RB_TYPE_P(vmax, T_FLOAT) && (v = rb_check_to_integer(vmax, "to_int"), !NIL_P(v))) {
-	v = rand_int(&rnd->mt, v, 1);
+	v = rand_int(obj, rnd, v, 1);
     }
     else if (v = rb_check_to_float(vmax), !NIL_P(v)) {
-	double max = float_value(v);
-	if (max > 0.0)
-	    v = rb_float_new(max * genrand_real(&rnd->mt));
-	else
+	max = float_value(v);
+	if (max < 0.0) {
 	    v = Qnil;
+	}
+	else {
+	    uint32_t a, b;
+	    double r;
+
+	  float_rand:
+	    a = random_int32(obj, rnd);
+	    b = random_int32(obj, rnd);
+	    r = int_pair_to_real_exclusive(a, b);
+	    if (max > 0.0) r *= max;;
+	    v = rb_float_new(r);
+	}
     }
-    else if ((v = rand_range(&rnd->mt, vmax)) != Qfalse) {
+    else if ((v = rand_range(obj, rnd, vmax)) != Qfalse) {
 	/* nothing to do */
     }
     else {
-	v = Qnil;
-	(void)NUM2LONG(vmax);
-    }
-    if (NIL_P(v)) {
-	VALUE mesg = rb_str_new_cstr("invalid argument - ");
-	rb_str_append(mesg, rb_obj_as_string(argv[0]));
-	rb_exc_raise(rb_exc_new3(rb_eArgError, mesg));
+	return Qfalse;
     }
+    return v;
+}
 
+static VALUE
+rand_random_number(int argc, VALUE *argv, VALUE obj)
+{
+    rb_random_t *rnd = try_get_rnd(obj);
+    VALUE v = rand_random(argc, argv, obj, rnd);
+    if (NIL_P(v)) v = rand_random(0, 0, obj, rnd);
+    else if (!v) invalid_argument(argv[0]);
     return v;
 }
 
@@ -1197,18 +1398,18 @@ static VALUE
 rb_f_rand(int argc, VALUE *argv, VALUE obj)
 {
     VALUE v, vmax, r;
-    struct MT *mt = default_mt();
+    rb_random_t *rnd = rand_start(&default_rand);
 
     if (argc == 0) goto zero_arg;
     rb_scan_args(argc, argv, "01", &vmax);
     if (NIL_P(vmax)) goto zero_arg;
-    if ((v = rand_range(mt, vmax)) != Qfalse) {
+    if ((v = rand_range(Qnil, rnd, vmax)) != Qfalse) {
 	return v;
     }
     vmax = rb_to_int(vmax);
-    if (vmax == INT2FIX(0) || NIL_P(r = rand_int(mt, vmax, 0))) {
+    if (vmax == INT2FIX(0) || NIL_P(r = rand_int(Qnil, rnd, vmax, 0))) {
       zero_arg:
-	return DBL2NUM(genrand_real(mt));
+	return DBL2NUM(genrand_real(&rnd->mt));
     }
     return r;
 }
@@ -1224,7 +1425,9 @@ rb_f_rand(int argc, VALUE *argv, VALUE obj)
 static VALUE
 random_s_rand(int argc, VALUE *argv, VALUE obj)
 {
-    return rand_random(argc, argv, rand_start(&default_rand));
+    VALUE v = rand_random(argc, argv, Qnil, rand_start(&default_rand));
+    check_random_number(v, argv);
+    return v;
 }
 
 #define SIP_HASH_STREAMING 0
@@ -1250,26 +1453,9 @@ static union {
     uint32_t u32[(16 * sizeof(uint8_t) - 1) / sizeof(uint32_t)];
 } sipseed;
 
-static VALUE
-init_randomseed(struct MT *mt, uint32_t initial[DEFAULT_SEED_CNT])
+static void
+init_hashseed(struct MT *mt)
 {
-    VALUE seed;
-    fill_random_seed(initial);
-    init_by_array(mt, initial, DEFAULT_SEED_CNT);
-    seed = make_seed_value(initial);
-    memset(initial, 0, DEFAULT_SEED_LEN);
-    return seed;
-}
-
-void
-Init_RandomSeed(void)
-{
-    rb_random_t *r = &default_rand;
-    uint32_t initial[DEFAULT_SEED_CNT];
-    struct MT *mt = &r->mt;
-    VALUE seed = init_randomseed(mt, initial);
-    int i;
-
     hashseed = genrand_int32(mt);
 #if SIZEOF_ST_INDEX_T*CHAR_BIT > 4*8
     hashseed <<= 32;
@@ -1283,12 +1469,15 @@ Init_RandomSeed(void)
     hashseed <<= 32;
     hashseed |= genrand_int32(mt);
 #endif
+}
+
+static void
+init_siphash(struct MT *mt)
+{
+    int i;
 
     for (i = 0; i < numberof(sipseed.u32); ++i)
 	sipseed.u32[i] = genrand_int32(mt);
-
-    rb_global_variable(&r->seed);
-    r->seed = seed;
 }
 
 st_index_t
@@ -1308,14 +1497,55 @@ rb_memhash(const void *ptr, long len)
 #endif
 }
 
-static void
-Init_RandomSeed2(void)
+/* Initialize Ruby internal seeds. This function is called at very early stage
+ * of Ruby startup. Thus, you can't use Ruby's object. */
+void
+Init_RandomSeedCore(void)
 {
-    VALUE seed = default_rand.seed;
+    /*
+      Don't reuse this MT for Random::DEFAULT. Random::DEFAULT::seed shouldn't
+      provide a hint that an attacker guess siphash's seed.
+    */
+    struct MT mt;
+    uint32_t initial_seed[DEFAULT_SEED_CNT];
 
-    if (RB_TYPE_P(seed, T_BIGNUM)) {
-	rb_obj_reveal(seed, rb_cBignum);
-    }
+    fill_random_seed(initial_seed);
+    init_by_array(&mt, initial_seed, DEFAULT_SEED_CNT);
+
+    init_hashseed(&mt);
+    init_siphash(&mt);
+
+    explicit_bzero(initial_seed, DEFAULT_SEED_LEN);
+}
+
+static VALUE
+init_randomseed(struct MT *mt)
+{
+    uint32_t initial[DEFAULT_SEED_CNT];
+    VALUE seed;
+
+    fill_random_seed(initial);
+    init_by_array(mt, initial, DEFAULT_SEED_CNT);
+    seed = make_seed_value(initial);
+    explicit_bzero(initial, DEFAULT_SEED_LEN);
+    return seed;
+}
+
+/* construct Random::DEFAULT bits */
+static VALUE
+Init_Random_default(void)
+{
+    rb_random_t *r = &default_rand;
+    struct MT *mt = &r->mt;
+    VALUE v;
+
+    r->seed = init_randomseed(mt);
+    rb_global_variable(&r->seed);
+
+    v = TypedData_Wrap_Struct(rb_cRandom, &random_data_type, r);
+    rb_gc_register_mark_object(v);
+
+    return v;
 }
 
 void
@@ -1353,7 +1583,6 @@ rb_reset_random_seed(void)
 void
 InitVM_Random(void)
 {
-    Init_RandomSeed2();
     rb_define_global_function("srand", rb_f_srand, -1);
     rb_define_global_function("rand", rb_f_rand, -1);
 
@@ -1371,17 +1600,23 @@ InitVM_Random(void)
     rb_define_method(rb_cRandom, "==", random_equal, 1);
 
     {
-	VALUE rand_default = TypedData_Wrap_Struct(rb_cRandom, &random_data_type, &default_rand);
-	rb_gc_register_mark_object(rand_default);
 	/* Direct access to Ruby's Pseudorandom number generator (PRNG). */
+	VALUE rand_default = Init_Random_default();
 	rb_define_const(rb_cRandom, "DEFAULT", rand_default);
     }
 
     rb_define_singleton_method(rb_cRandom, "srand", rb_f_srand, -1);
     rb_define_singleton_method(rb_cRandom, "rand", random_s_rand, -1);
     rb_define_singleton_method(rb_cRandom, "new_seed", random_seed, 0);
+    rb_define_singleton_method(rb_cRandom, "raw_seed", random_raw_seed, 1);
     rb_define_private_method(CLASS_OF(rb_cRandom), "state", random_s_state, 0);
     rb_define_private_method(CLASS_OF(rb_cRandom), "left", random_s_left, 0);
+
+    {
+	VALUE m = rb_define_module_under(rb_cRandom, "Formatter");
+	rb_include_module(rb_cRandom, m);
+	rb_define_method(m, "random_number", rand_random_number, -1);
+    }
 }
 
 #undef rb_intern
