@@ -2,7 +2,7 @@
 
   re.c -
 
-  $Author: nobu $
+  $Author$
   created at: Mon Aug  9 18:24:49 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
