@@ -1,6 +1,6 @@
+# frozen_string_literal: false
 require 'test/unit'
 require "-test-/struct"
-require_relative '../../ruby/envutil'
 
 class  Bug::Struct::Test_Member < Test::Unit::TestCase
   S = Bug::Struct.new(:a)
