@@ -1,3 +1,4 @@
+# frozen_string_literal: false
 require_relative "testbase"
 require 'bigdecimal/math'
 
@@ -458,18 +459,6 @@ class TestBigDecimal < Test::Unit::TestCase
     assert_equal(false, BigDecimal.new("NaN") > n1)
   end
 
-  def test_cmp_float_nan
-    assert_equal(nil, BigDecimal.new("1") <=> Float::NAN)
-  end
-
-  def test_cmp_float_pos_inf
-    assert_equal(-1, BigDecimal.new("1") <=> Float::INFINITY)
-  end
-
-  def test_cmp_float_neg_inf
-    assert_equal(+1, BigDecimal.new("1") <=> -Float::INFINITY)
-  end
-
   def test_cmp_failing_coercion
     n1 = BigDecimal.new("1")
     assert_equal(nil, n1 <=> nil)
@@ -1564,4 +1553,31 @@ class TestBigDecimal < Test::Unit::TestCase
     Thread.current.keys.to_s
     EOS
   end
+
+  def assert_no_memory_leak(code, *rest, **opt)
+    code = "8.times {20_000.times {begin #{code}; rescue NoMemoryError; end}; GC.start}"
+    super(["-rbigdecimal"],
+          "b = BigDecimal('10'); b.nil?; " \
+          "GC.add_stress_to_class(BigDecimal); "\
+          "#{code}", code, *rest, rss: true, limit: 1.1, **opt)
+  end
+
+  if EnvUtil.gc_stress_to_class?
+    def test_no_memory_leak_allocate
+      assert_no_memory_leak("BigDecimal.allocate")
+    end
+
+    def test_no_memory_leak_initialize
+      assert_no_memory_leak("BigDecimal.new")
+    end
+
+    def test_no_memory_leak_global_new
+      assert_no_memory_leak("BigDecimal('10')")
+      assert_no_memory_leak("BigDecimal(b)")
+    end
+
+    def test_no_memory_leak_create
+      assert_no_memory_leak("b + 10")
+    end
+  end
 end
