@@ -1,3 +1,4 @@
+# frozen_string_literal: false
 begin
   gem 'minitest', '< 5.0.0' if defined? Gem
 rescue Gem::LoadError
@@ -61,13 +62,9 @@ module Test
         orig_args -= args
         args = @init_hook.call(args, options) if @init_hook
         non_options(args, options)
+        @run_options = orig_args
         @help = orig_args.map { |s| s =~ /[\s|&<>$()]/ ? s.inspect : s }.join " "
         @options = options
-        if @options[:parallel]
-          @files = args
-          @args = orig_args
-        end
-        options
       end
 
       private
@@ -75,10 +72,6 @@ module Test
         opts.separator 'minitest options:'
         opts.version = MiniTest::Unit::VERSION
 
-        options[:retry] = true
-        options[:job_status] = nil
-        options[:hide_skip] = true
-
         opts.on '-h', '--help', 'Display this help.' do
           puts opts
           exit
@@ -93,14 +86,43 @@ module Test
           self.verbose = options[:verbose]
         end
 
-        opts.on '-n', '--name PATTERN', "Filter test names on pattern." do |a|
+        opts.on '-n', '--name PATTERN', "Filter test method names on pattern: /REGEXP/ or STRING" do |a|
           options[:filter] = a
         end
 
-        opts.on '--jobs-status [TYPE]', [:normal, :replace],
-                "Show status of jobs every file; Disabled when --jobs isn't specified." do |type|
-          options[:job_status] = type || :normal
+        opts.on '--test-order=random|alpha|sorted', [:random, :alpha, :sorted] do |a|
+          MiniTest::Unit::TestCase.test_order = a
         end
+      end
+
+      def non_options(files, options)
+        true
+      end
+    end
+
+    module Parallel # :nodoc: all
+      def process_args(args = [])
+        return @options if @options
+        options = super
+        if @options[:parallel]
+          @files = args
+        end
+        options
+      end
+
+      def status(*args)
+        result = super
+        raise @interrupt if @interrupt
+        result
+      end
+
+      private
+      def setup_options(opts, options)
+        super
+
+        opts.separator "parallel test options:"
+
+        options[:retry] = true
 
         opts.on '-j N', '--jobs N', "Allow run tests with N jobs at once" do |a|
           if /^t/ =~ a
@@ -127,153 +149,8 @@ module Test
         opts.on '--ruby VAL', "Path to ruby; It'll have used at -j option" do |a|
           options[:ruby] = a.split(/ /).reject(&:empty?)
         end
-
-        opts.on '-q', '--hide-skip', 'Hide skipped tests' do
-          options[:hide_skip] = true
-        end
-
-        opts.on '--show-skip', 'Show skipped tests' do
-          options[:hide_skip] = false
-        end
-
-        opts.on '--color[=WHEN]',
-                [:always, :never, :auto],
-                "colorize the output.  WHEN defaults to 'always'", "or can be 'never' or 'auto'." do |c|
-          options[:color] = c || :always
-        end
-
-        opts.on '--tty[=WHEN]',
-                [:yes, :no],
-                "force to output tty control.  WHEN defaults to 'yes'", "or can be 'no'." do |c|
-          @tty = c != :no
-        end
       end
 
-      def non_options(files, options)
-        begin
-          require "rbconfig"
-        rescue LoadError
-          warn "#{caller(1)[0]}: warning: Parallel running disabled because can't get path to ruby; run specify with --ruby argument"
-          options[:parallel] = nil
-        else
-          options[:ruby] ||= [RbConfig.ruby]
-        end
-
-        true
-      end
-    end
-
-    module GlobOption # :nodoc: all
-      @@testfile_prefix = "test"
-
-      def setup_options(parser, options)
-        super
-        parser.on '-b', '--basedir=DIR', 'Base directory of test suites.' do |dir|
-          options[:base_directory] = dir
-        end
-        parser.on '-x', '--exclude PATTERN', 'Exclude test files on pattern.' do |pattern|
-          (options[:reject] ||= []) << pattern
-        end
-      end
-
-      def non_options(files, options)
-        paths = [options.delete(:base_directory), nil].uniq
-        if reject = options.delete(:reject)
-          reject_pat = Regexp.union(reject.map {|r| /#{r}/ })
-        end
-        files.map! {|f|
-          f = f.tr(File::ALT_SEPARATOR, File::SEPARATOR) if File::ALT_SEPARATOR
-          ((paths if /\A\.\.?(?:\z|\/)/ !~ f) || [nil]).any? do |prefix|
-            if prefix
-              path = f.empty? ? prefix : "#{prefix}/#{f}"
-            else
-              next if f.empty?
-              path = f
-            end
-            if !(match = Dir["#{path}/**/#{@@testfile_prefix}_*.rb"]).empty?
-              if reject
-                match.reject! {|n|
-                  n[(prefix.length+1)..-1] if prefix
-                  reject_pat =~ n
-                }
-              end
-              break match
-            elsif !reject or reject_pat !~ f and File.exist? path
-              break path
-            end
-          end or
-            raise ArgumentError, "file not found: #{f}"
-        }
-        files.flatten!
-        super(files, options)
-      end
-    end
-
-    module LoadPathOption # :nodoc: all
-      def setup_options(parser, options)
-        super
-        parser.on '-Idirectory', 'Add library load path' do |dirs|
-          dirs.split(':').each { |d| $LOAD_PATH.unshift d }
-        end
-      end
-    end
-
-    module GCStressOption # :nodoc: all
-      def setup_options(parser, options)
-        super
-        parser.on '--[no-]gc-stress', 'Set GC.stress as true' do |flag|
-          options[:gc_stress] = flag
-        end
-      end
-
-      def non_options(files, options)
-        if options.delete(:gc_stress)
-          MiniTest::Unit::TestCase.class_eval do
-            oldrun = instance_method(:run)
-            define_method(:run) do |runner|
-              begin
-                gc_stress, GC.stress = GC.stress, true
-                oldrun.bind(self).call(runner)
-              ensure
-                GC.stress = gc_stress
-              end
-            end
-          end
-        end
-        super
-      end
-    end
-
-    module RequireFiles # :nodoc: all
-      def non_options(files, options)
-        return false if !super
-        errors = {}
-        result = false
-        files.each {|f|
-          d = File.dirname(path = File.realpath(f))
-          unless $:.include? d
-            $: << d
-          end
-          begin
-            require path unless options[:parallel]
-            result = true
-          rescue LoadError
-            next if errors[$!.message]
-            errors[$!.message] = true
-            puts "#{f}: #{$!}"
-          end
-        }
-        result
-      end
-    end
-
-    class Runner < MiniTest::Unit # :nodoc: all
-      include Test::Unit::Options
-      include Test::Unit::GlobOption
-      include Test::Unit::LoadPathOption
-      include Test::Unit::GCStressOption
-      include Test::Unit::RunCount
-
       class Worker
         def self.launch(ruby,args=[])
           io = IO.popen([*ruby,
@@ -347,6 +224,10 @@ module Test
         def died(*additional)
           @status = :quit
           @io.close
+          status = $?
+          if status and status.signaled?
+            additional[0] ||= SignalException.new(status.termsig)
+          end
 
           call_hook(:dead,*additional)
         end
@@ -372,18 +253,6 @@ module Test
 
       end
 
-      class << self; undef autorun; end
-
-      @@stop_auto_run = false
-      def self.autorun
-        at_exit {
-          Test::Unit::RunCount.run_once {
-            exit(Test::Unit::Runner.new.run(ARGV) || true)
-          } unless @@stop_auto_run
-        } unless @@installed_at_exit
-        @@installed_at_exit = true
-      end
-
       def after_worker_down(worker, e=nil, c=false)
         return unless @options[:parallel]
         return if @interrupt
@@ -398,69 +267,6 @@ module Test
         exit c
       end
 
-      def terminal_width
-        unless @terminal_width ||= nil
-          begin
-            require 'io/console'
-            width = $stdout.winsize[1]
-          rescue LoadError, NoMethodError, Errno::ENOTTY, Errno::EBADF, Errno::EINVAL
-            width = ENV["COLUMNS"].to_i.nonzero? || 80
-          end
-          width -= 1 if /mswin|mingw/ =~ RUBY_PLATFORM
-          @terminal_width = width
-        end
-        @terminal_width
-      end
-
-      def del_status_line
-        @status_line_size ||= 0
-        unless @options[:job_status] == :replace
-          $stdout.puts
-          return
-        end
-        print "\r"+" "*@status_line_size+"\r"
-        $stdout.flush
-        @status_line_size = 0
-      end
-
-      def put_status(line)
-        unless @options[:job_status] == :replace
-          print(line)
-          return
-        end
-        @status_line_size ||= 0
-        del_status_line
-        $stdout.flush
-        line = line[0...terminal_width]
-        print line
-        $stdout.flush
-        @status_line_size = line.size
-      end
-
-      def add_status(line)
-        unless @options[:job_status] == :replace
-          print(line)
-          return
-        end
-        @status_line_size ||= 0
-        line = line[0...(terminal_width-@status_line_size)]
-        print line
-        $stdout.flush
-        @status_line_size += line.size
-      end
-
-      def jobs_status
-        return unless @options[:job_status]
-        puts "" unless @options[:verbose] or @options[:job_status] == :replace
-        status_line = @workers.map(&:to_s).join(" ")
-        update_status(status_line) or (puts; nil)
-      end
-
-      def del_jobs_status
-        return unless @options[:job_status] == :replace && @status_line_size.nonzero?
-        del_status_line
-      end
-
       def after_worker_quit(worker)
         return unless @options[:parallel]
         return if @interrupt
@@ -471,7 +277,7 @@ module Test
 
       def launch_worker
         begin
-          worker = Worker.launch(@options[:ruby],@args)
+          worker = Worker.launch(@options[:ruby], @run_options)
         rescue => e
           abort "ERROR: Failed to launch job process - #{e.class}: #{e.message}"
         end
@@ -495,7 +301,7 @@ module Test
         return if @workers.empty?
         @workers.reject! do |worker|
           begin
-            timeout(1) do
+            Timeout.timeout(1) do
               worker.quit
             end
           rescue Errno::EPIPE
@@ -506,7 +312,7 @@ module Test
 
         return if @workers.empty?
         begin
-          timeout(0.2 * @workers.size) do
+          Timeout.timeout(0.2 * @workers.size) do
             Process.waitall
           end
         rescue Timeout::Error
@@ -517,26 +323,10 @@ module Test
         end
       end
 
-      def start_watchdog
-        Thread.new do
-          while stat = Process.wait2
-            break if @interrupt # Break when interrupt
-            pid, stat = stat
-            w = (@workers + @dead_workers).find{|x| pid == x.pid }
-            next unless w
-            w = w.dup
-            if w.status != :quit && !w.quit_called?
-              # Worker down
-              w.died(nil, !stat.signaled? && stat.exitstatus)
-            end
-          end
-        end
-      end
-
       def deal(io, type, result, rep, shutting_down = false)
         worker = @workers_hash[io]
         cmd = worker.read
-        cmd.sub!(/\A\.+/, '')
+        cmd.sub!(/\A\.+/, '') if cmd # read may return nil
         case cmd
         when ''
           # just only dots, ignore
@@ -607,9 +397,6 @@ module Test
         @workers_hash = {} # out-IO => worker
         @ios          = [] # Array of worker IOs
         begin
-          # Thread: watchdog
-          watchdog = start_watchdog
-
           @options[:parallel].times {launch_worker}
 
           while _io = IO.select(@ios)[0]
@@ -623,7 +410,6 @@ module Test
           @interrupt = ex
           return result
         ensure
-          watchdog.kill if watchdog
           if @interrupt
             @ios.select!{|x| @workers_hash[x].status == :running }
             while !@ios.empty? && (__io = IO.select(@ios,[],[],10))
@@ -687,13 +473,87 @@ module Test
             end
           }
         end
+        result
+      end
+    end
+
+    module Skipping # :nodoc: all
+      private
+      def setup_options(opts, options)
+        super
+
+        opts.separator "skipping options:"
+
+        options[:hide_skip] = true
+
+        opts.on '-q', '--hide-skip', 'Hide skipped tests' do
+          options[:hide_skip] = true
+        end
+
+        opts.on '--show-skip', 'Show skipped tests' do
+          options[:hide_skip] = false
+        end
+      end
+
+      private
+      def _run_suites(suites, type)
+        result = super
         report.reject!{|r| r.start_with? "Skipped:" } if @options[:hide_skip]
         report.sort_by!{|r| r.start_with?("Skipped:") ? 0 : \
                            (r.start_with?("Failure:") ? 1 : 2) }
         result
       end
+    end
 
-      alias mini_run_suite _run_suite
+    module StatusLine # :nodoc: all
+      def terminal_width
+        unless @terminal_width ||= nil
+          begin
+            require 'io/console'
+            width = $stdout.winsize[1]
+          rescue LoadError, NoMethodError, Errno::ENOTTY, Errno::EBADF, Errno::EINVAL
+            width = ENV["COLUMNS"].to_i.nonzero? || 80
+          end
+          width -= 1 if /mswin|mingw/ =~ RUBY_PLATFORM
+          @terminal_width = width
+        end
+        @terminal_width
+      end
+
+      def del_status_line(flush = true)
+        @status_line_size ||= 0
+        unless @options[:job_status] == :replace
+          $stdout.puts
+          return
+        end
+        print "\r"+" "*@status_line_size+"\r"
+        $stdout.flush if flush
+        @status_line_size = 0
+      end
+
+      def add_status(line, flush: true)
+        unless @options[:job_status] == :replace
+          print(line)
+          return
+        end
+        @status_line_size ||= 0
+        line = line[0...(terminal_width-@status_line_size)]
+        print line
+        $stdout.flush if flush
+        @status_line_size += line.size
+      end
+
+      def jobs_status
+        return unless @options[:job_status]
+        puts "" unless @options[:verbose] or @options[:job_status] == :replace
+        status_line = @workers.map(&:to_s).join(" ")
+        update_status(status_line) or (puts; nil)
+      end
+
+      def del_jobs_status
+        return unless @options[:job_status] == :replace && @status_line_size.nonzero?
+        del_status_line
+      end
 
       def output
         (@output ||= nil) || super
@@ -705,23 +565,29 @@ module Test
         when :always
           color = true
         when :auto, nil
-          color = @options[:job_status] == :replace && /dumb/ !~ ENV["TERM"]
+          color = (@tty || @options[:job_status] == :replace) && /dumb/ !~ ENV["TERM"]
         else
           color = false
         end
         if color
           # dircolors-like style
-          colors = (colors = ENV['TEST_COLORS']) ? Hash[colors.scan(/(\w+)=([^:]*)/)] : {}
-          @passed_color = "\e[#{colors["pass"] || "32"}m"
-          @failed_color = "\e[#{colors["fail"] || "31"}m"
-          @skipped_color = "\e[#{colors["skip"] || "33"}m"
+          colors = (colors = ENV['TEST_COLORS']) ? Hash[colors.scan(/(\w+)=([^:\n]*)/)] : {}
+          begin
+            File.read(File.join(__dir__, "../../colors")).scan(/(\w+)=([^:\n]*)/) do |n, c|
+              colors[n] ||= c
+            end
+          rescue
+          end
+          @passed_color = "\e[;#{colors["pass"] || "32"}m"
+          @failed_color = "\e[;#{colors["fail"] || "31"}m"
+          @skipped_color = "\e[;#{colors["skip"] || "33"}m"
           @reset_color = "\e[m"
         else
           @passed_color = @failed_color = @skipped_color = @reset_color = ""
         end
         if color or @options[:job_status] == :replace
           @verbose = !options[:parallel]
-          @output = StatusLineOutput.new(self)
+          @output = Output.new(self)
         end
         if /\A\/(.*)\/\z/ =~ (filter = options[:filter])
           filter = Regexp.new($1)
@@ -743,7 +609,11 @@ module Test
 
       def update_status(s)
         count = @test_count.to_s(10).rjust(@total_tests.size)
-        put_status("#{@passed_color}[#{count}/#{@total_tests}]#{@reset_color} #{s}")
+        del_status_line(false) if @options[:job_status] == :replace
+        print(@passed_color)
+        add_status("[#{count}/#{@total_tests}]", flush: false)
+        print(@reset_color)
+        add_status(" #{s}")
       end
 
       def _print(s); $stdout.print(s); end
@@ -771,6 +641,272 @@ module Test
         report.clear
       end
 
+      def initialize
+        super
+        @tty = $stdout.tty?
+      end
+
+      def run(*args)
+        result = super
+        puts "\nruby -v: #{RUBY_DESCRIPTION}"
+        result
+      end
+
+      private
+      def setup_options(opts, options)
+        super
+
+        opts.separator "status line options:"
+
+        options[:job_status] = nil
+
+        opts.on '--jobs-status [TYPE]', [:normal, :replace],
+                "Show status of jobs every file; Disabled when --jobs isn't specified." do |type|
+          options[:job_status] = type || :normal
+        end
+
+        opts.on '--color[=WHEN]',
+                [:always, :never, :auto],
+                "colorize the output.  WHEN defaults to 'always'", "or can be 'never' or 'auto'." do |c|
+          options[:color] = c || :always
+        end
+
+        opts.on '--tty[=WHEN]',
+                [:yes, :no],
+                "force to output tty control.  WHEN defaults to 'yes'", "or can be 'no'." do |c|
+          @tty = c != :no
+        end
+      end
+
+      class Output < Struct.new(:runner) # :nodoc: all
+        def puts(*a) $stdout.puts(*a) unless a.empty? end
+        def respond_to_missing?(*a) $stdout.respond_to?(*a) end
+        def method_missing(*a, &b) $stdout.__send__(*a, &b) end
+
+        def print(s)
+          case s
+          when /\A(.*\#.*) = \z/
+            runner.new_test($1)
+          when /\A(.* s) = \z/
+            runner.add_status(" = "+$1.chomp)
+          when /\A\.+\z/
+            runner.succeed
+          when /\A[EFS]\z/
+            runner.failed(s)
+          else
+            $stdout.print(s)
+          end
+        end
+      end
+    end
+
+    module LoadPathOption # :nodoc: all
+      def non_options(files, options)
+        begin
+          require "rbconfig"
+        rescue LoadError
+          warn "#{caller(1)[0]}: warning: Parallel running disabled because can't get path to ruby; run specify with --ruby argument"
+          options[:parallel] = nil
+        else
+          options[:ruby] ||= [RbConfig.ruby]
+        end
+
+        super
+      end
+
+      def setup_options(parser, options)
+        super
+        parser.separator "load path options:"
+        parser.on '-Idirectory', 'Add library load path' do |dirs|
+          dirs.split(':').each { |d| $LOAD_PATH.unshift d }
+        end
+      end
+    end
+
+    module GlobOption # :nodoc: all
+      @@testfile_prefix = "test"
+
+      def setup_options(parser, options)
+        super
+        parser.separator "globbing options:"
+        parser.on '-b', '--basedir=DIR', 'Base directory of test suites.' do |dir|
+          options[:base_directory] = dir
+        end
+        parser.on '-x', '--exclude REGEXP', 'Exclude test files on pattern.' do |pattern|
+          (options[:reject] ||= []) << pattern
+        end
+      end
+
+      def non_options(files, options)
+        paths = [options.delete(:base_directory), nil].uniq
+        if reject = options.delete(:reject)
+          reject_pat = Regexp.union(reject.map {|r| %r"#{r}"})
+        end
+        files.map! {|f|
+          f = f.tr(File::ALT_SEPARATOR, File::SEPARATOR) if File::ALT_SEPARATOR
+          ((paths if /\A\.\.?(?:\z|\/)/ !~ f) || [nil]).any? do |prefix|
+            if prefix
+              path = f.empty? ? prefix : "#{prefix}/#{f}"
+            else
+              next if f.empty?
+              path = f
+            end
+            if !(match = Dir["#{path}/**/#{@@testfile_prefix}_*.rb"]).empty?
+              if reject
+                match.reject! {|n|
+                  n[(prefix.length+1)..-1] if prefix
+                  reject_pat =~ n
+                }
+              end
+              break match
+            elsif !reject or reject_pat !~ f and File.exist? path
+              break path
+            end
+          end or
+            raise ArgumentError, "file not found: #{f}"
+        }
+        files.flatten!
+        super(files, options)
+      end
+    end
+
+    module GCStressOption # :nodoc: all
+      def setup_options(parser, options)
+        super
+        parser.separator "GC options:"
+        parser.on '--[no-]gc-stress', 'Set GC.stress as true' do |flag|
+          options[:gc_stress] = flag
+        end
+      end
+
+      def non_options(files, options)
+        if options.delete(:gc_stress)
+          MiniTest::Unit::TestCase.class_eval do
+            oldrun = instance_method(:run)
+            define_method(:run) do |runner|
+              begin
+                gc_stress, GC.stress = GC.stress, true
+                oldrun.bind(self).call(runner)
+              ensure
+                GC.stress = gc_stress
+              end
+            end
+          end
+        end
+        super
+      end
+    end
+
+    module RequireFiles # :nodoc: all
+      def non_options(files, options)
+        return false if !super
+        errors = {}
+        result = false
+        files.each {|f|
+          d = File.dirname(path = File.realpath(f))
+          unless $:.include? d
+            $: << d
+          end
+          begin
+            require path unless options[:parallel]
+            result = true
+          rescue LoadError
+            next if errors[$!.message]
+            errors[$!.message] = true
+            puts "#{f}: #{$!}"
+          end
+        }
+        result
+      end
+    end
+
+    module ExcludesOption # :nodoc: all
+      class ExcludedMethods < Struct.new(:excludes)
+        def exclude(name, reason)
+          excludes[name] = reason
+        end
+
+        def exclude_from(klass)
+          excludes = self.excludes
+          pattern = excludes.keys.grep(Regexp).tap {|k|
+            break (Regexp.new(k.join('|')) unless k.empty?)
+          }
+          klass.class_eval do
+            public_instance_methods(false).each do |method|
+              if excludes[method] or (pattern and pattern =~ method)
+                remove_method(method)
+              end
+            end
+            public_instance_methods(true).each do |method|
+              if excludes[method] or (pattern and pattern =~ method)
+                undef_method(method)
+              end
+            end
+          end
+        end
+
+        def self.load(dirs, name)
+          return unless dirs and name
+          instance = nil
+          dirs.each do |dir|
+            path = File.join(dir, name.gsub(/::/, '/') + ".rb")
+            begin
+              src = File.read(path)
+            rescue Errno::ENOENT
+              nil
+            else
+              instance ||= new({})
+              instance.instance_eval(src)
+            end
+          end
+          instance
+        end
+      end
+
+      def setup_options(parser, options)
+        super
+        if excludes = ENV["EXCLUDES"]
+          excludes = excludes.split(File::PATH_SEPARATOR)
+        end
+        options[:excludes] = excludes || []
+        parser.on '-X', '--excludes-dir DIRECTORY', "Directory name of exclude files" do |d|
+          options[:excludes].concat d.split(File::PATH_SEPARATOR)
+        end
+      end
+
+      def _run_suite(suite, type)
+        if ex = ExcludedMethods.load(@options[:excludes], suite.name)
+          ex.exclude_from(suite)
+        end
+        super
+      end
+    end
+
+    class Runner < MiniTest::Unit # :nodoc: all
+      include Test::Unit::Options
+      include Test::Unit::StatusLine
+      include Test::Unit::Parallel
+      include Test::Unit::Skipping
+      include Test::Unit::GlobOption
+      include Test::Unit::LoadPathOption
+      include Test::Unit::GCStressOption
+      include Test::Unit::ExcludesOption
+      include Test::Unit::RunCount
+
+      class << self; undef autorun; end
+
+      @@stop_auto_run = false
+      def self.autorun
+        at_exit {
+          Test::Unit::RunCount.run_once {
+            exit(Test::Unit::Runner.new.run(ARGV) || true)
+          } unless @@stop_auto_run
+        } unless @@installed_at_exit
+        @@installed_at_exit = true
+      end
+
+      alias mini_run_suite _run_suite
+
       # Overriding of MiniTest::Unit#puke
       def puke klass, meth, e
         # TODO:
@@ -784,44 +920,6 @@ module Test
         end
         rep
       end
-
-      def initialize
-        super
-        @tty = $stdout.tty?
-      end
-
-      def status(*args)
-        result = super
-        raise @interrupt if @interrupt
-        result
-      end
-
-      def run(*args)
-        result = super
-        puts "\nruby -v: #{RUBY_DESCRIPTION}"
-        result
-      end
-    end
-
-    class StatusLineOutput < Struct.new(:runner) # :nodoc: all
-      def puts(*a) $stdout.puts(*a) unless a.empty? end
-      def respond_to_missing?(*a) $stdout.respond_to?(*a) end
-      def method_missing(*a, &b) $stdout.__send__(*a, &b) end
-
-      def print(s)
-        case s
-        when /\A(.*\#.*) = \z/
-          runner.new_test($1)
-        when /\A(.* s) = \z/
-          runner.add_status(" = "+$1.chomp)
-        when /\A\.+\z/
-          runner.succeed
-        when /\A[EFS]\z/
-          runner.failed(s)
-        else
-          $stdout.print(s)
-        end
-      end
     end
 
     class AutoRunner # :nodoc: all
@@ -882,6 +980,15 @@ module MiniTest # :nodoc: all
 end
 
 class MiniTest::Unit::TestCase # :nodoc: all
+  test_order = self.test_order
+  class << self
+    attr_writer :test_order
+    undef test_order
+  end
+  def self.test_order
+    defined?(@test_order) ? @test_order : superclass.test_order
+  end
+  self.test_order = test_order
   undef run_test
   RUN_TEST_TRACE = "#{__FILE__}:#{__LINE__+3}:in `run_test'".freeze
   def run_test(name)
