@@ -1,3 +1,4 @@
+# frozen_string_literal: false
 require_relative 'utils'
 
 class  OpenSSL::TestASN1 < Test::Unit::TestCase
@@ -200,7 +201,7 @@ class  OpenSSL::TestASN1 < Test::Unit::TestCase
 
   def test_encode_nil
     m = OpenSSL::ASN1
-    [ 
+    [
       m::Boolean, m::Integer, m::BitString, m::OctetString,
       m::ObjectId, m::Enumerated, m::UTF8String, m::UTCTime,
       m::GeneralizedTime, m::Sequence, m::Set
@@ -247,7 +248,7 @@ rEzBQ0F9dUyqQ9gyRg8KHhDfv9HzT1d/rnUZMkoombwYBRIUChGCYV0GnJcan2Zm
       assert_equal(false, prim.infinite_length)
       prim.infinite_length = true
       flunk('Could set infinite length on primitive value')
-    rescue NoMethodError => e
+    rescue NoMethodError
       #ok
     end
   end
@@ -263,6 +264,14 @@ rEzBQ0F9dUyqQ9gyRg8KHhDfv9HzT1d/rnUZMkoombwYBRIUChGCYV0GnJcan2Zm
     end
   end
 
+  def test_decode_utctime
+    expected = Time.at 1374535380
+    assert_equal expected, OpenSSL::ASN1.decode("\x17\v1307222323Z").value
+
+    expected += 17
+    assert_equal expected, OpenSSL::ASN1.decode("\x17\r130722232317Z").value
+  end
+
   def test_create_inf_length_primitive
     expected = %w{ 24 80 04 01 61 00 00 }
     raw = [expected.join('')].pack('H*')
@@ -544,7 +553,7 @@ rEzBQ0F9dUyqQ9gyRg8KHhDfv9HzT1d/rnUZMkoombwYBRIUChGCYV0GnJcan2Zm
   end
 
   def test_primitive_inf_length
-    assert_raises(OpenSSL::ASN1::ASN1Error) do
+    assert_raise(OpenSSL::ASN1::ASN1Error) do
       spec = %w{ 02 80 02 01 01 00 00 }
       raw = [spec.join('')].pack('H*')
       OpenSSL::ASN1.decode(raw)
@@ -597,5 +606,5 @@ rEzBQ0F9dUyqQ9gyRg8KHhDfv9HzT1d/rnUZMkoombwYBRIUChGCYV0GnJcan2Zm
     assert_equal(:UNIVERSAL, asn1.tag_class)
   end
 
-end if defined?(OpenSSL)
+end if defined?(OpenSSL::TestUtils)
 
