@@ -1,11 +1,11 @@
-require 'pp'
-require 'rubygems'
-require 'minitest/autorun'
-require 'rdoc/markup'
+# frozen_string_literal: false
+require 'rdoc/test_case'
 
-class TestRDocMarkupIndentedParagraph < MiniTest::Unit::TestCase
+class TestRDocMarkupIndentedParagraph < RDoc::TestCase
 
   def setup
+    super
+
     @IP = RDoc::Markup::IndentedParagraph
   end
 
@@ -36,5 +36,19 @@ class TestRDocMarkupIndentedParagraph < MiniTest::Unit::TestCase
     refute_equal one, two
   end
 
+  def test_text
+    paragraph = @IP.new(2, 'hello', ' world')
+
+    assert_equal 'hello world', paragraph.text
+  end
+
+  def test_text_break
+    paragraph = @IP.new(2, 'hello', hard_break, 'world')
+
+    assert_equal 'helloworld', paragraph.text
+
+    assert_equal "hello\n  world", paragraph.text("\n")
+  end
+
 end
 
