@@ -1,3 +1,4 @@
+# frozen_string_literal: false
 require 'test/unit'
 require 'tempfile'
 
@@ -180,16 +181,6 @@ class TestException < Test::Unit::TestCase
     }
   end
 
-  def test_throw_false
-    bug12743 = '[ruby-core:77229] [Bug #12743]'
-    e = assert_raise_with_message(UncaughtThrowError, /false/, bug12743) {
-      Thread.start {
-        throw false
-      }.join
-    }
-    assert_same(false, e.tag, bug12743)
-  end
-
   def test_else_no_exception
     begin
       assert(true)
@@ -312,6 +303,10 @@ class TestException < Test::Unit::TestCase
     assert_raise_with_message(TypeError, /C\u{4032}/) do
       [*o]
     end
+    obj = eval("class C\u{1f5ff}; self; end").new
+    assert_raise_with_message(TypeError, /C\u{1f5ff}/) do
+      Class.new {include obj}
+    end
   end
 
   def test_errat
@@ -435,9 +430,11 @@ end.join
     bug3237 = '[ruby-core:29948]'
     str = "\u2600"
     id = :"\u2604"
-    msg = "undefined method `#{id}' for #{str.inspect}:String"
-    assert_raise_with_message(NoMethodError, msg, bug3237) do
-      str.__send__(id)
+    EnvUtil.with_default_external(Encoding::UTF_8) do
+      msg = "undefined method `#{id}' for #{str.inspect}:String"
+      assert_raise_with_message(NoMethodError, msg, bug3237) do
+        str.__send__(id)
+      end
     end
   end
 
@@ -628,41 +625,6 @@ end.join
     assert_not_same(e, e.cause, "#{msg}: should not be recursive")
   end
 
-  def test_cause_raised_in_rescue
-    e = assert_raise_with_message(RuntimeError, 'b') {
-      begin
-        raise 'a'
-      rescue => a
-        begin
-          raise 'b'
-        rescue => b
-          begin
-            raise 'c'
-          rescue
-            raise b
-          end
-        end
-      end
-    }
-    assert_equal('a', e.cause.message, 'cause should not be overwritten by reraise')
-  end
-
-  def test_cause_at_raised
-    e = assert_raise_with_message(RuntimeError, 'b') {
-      begin
-        raise 'a'
-      rescue => a
-        b = RuntimeError.new('b')
-        begin
-          raise 'c'
-        rescue
-          raise b
-        end
-      end
-    }
-    assert_equal('c', e.cause.message, 'cause should be the exception at raised')
-  end
-
   def test_raise_with_cause
     msg = "[Feature #8257]"
     cause = ArgumentError.new("foobar")
@@ -677,25 +639,6 @@ end.join
     end
   end
 
-  def test_raise_with_cause_in_rescue
-    e = assert_raise_with_message(RuntimeError, 'b') {
-      begin
-        raise 'a'
-      rescue => a
-        begin
-          raise 'b'
-        rescue => b
-          begin
-            raise 'c'
-          rescue
-            raise b, cause: ArgumentError.new('d')
-          end
-        end
-      end
-    }
-    assert_equal('d', e.cause.message, 'cause option should be honored always')
-  end
-
   def test_unknown_option
     bug = '[ruby-core:63203] [Feature #8257] should pass unknown options'
 
@@ -723,19 +666,39 @@ end.join
 
   def test_name_error_info
     obj = BasicObject.new
+    class << obj
+      alias object_id __id__
+      def pretty_inspect; "`obj'"; end
+    end
     e = assert_raise(NameError) {
       obj.instance_eval("Object")
     }
     assert_equal(:Object, e.name)
+    e = assert_raise(NameError) {
+      BasicObject::X
+    }
+    assert_same(BasicObject, e.receiver)
     e = assert_raise(NameError) {
       obj.instance_eval {foo}
     }
     assert_equal(:foo, e.name)
+    assert_same(obj, e.receiver)
     e = assert_raise(NoMethodError) {
       obj.foo(1, 2)
     }
     assert_equal(:foo, e.name)
     assert_equal([1, 2], e.args)
+    assert_same(obj, e.receiver)
+    def obj.test(a, b=nil, *c, &d)
+      e = a
+      1.times {|f| g = foo}
+    end
+    e = assert_raise(NameError) {
+      obj.test(3)
+    }
+    assert_equal(:foo, e.name)
+    assert_same(obj, e.receiver)
+    assert_equal(%i[a b c d e f g], e.local_variables.sort)
   end
 
   def test_output_string_encoding
@@ -770,50 +733,11 @@ $stderr = $stdout; raise "\x82\xa0"') do |outs, errs, status|
     assert_raise(NoMethodError, bug10969) {a.public_send("bar", true)}
   end
 
-  def test_undefined_backtrace
-    assert_separately([], "#{<<-"begin;"}\n#{<<-"end;"}")
-    begin;
-      class Exception
-        undef backtrace
+  def test_message_of_name_error
+    assert_raise_with_message(NameError, /\Aundefined method `foo' for module `#<Module:.*>'$/) do
+      Module.new do
+        module_function :foo
       end
-
-      assert_raise(RuntimeError) {
-        raise RuntimeError, "hello"
-      }
-    end;
-  end
-
-  def test_redefined_backtrace
-    assert_separately([], "#{<<-"begin;"}\n#{<<-"end;"}")
-    begin;
-      $exc = nil
-
-      class Exception
-        undef backtrace
-        def backtrace
-          $exc = self
-        end
-      end
-
-      e = assert_raise(RuntimeError) {
-        raise RuntimeError, "hello"
-      }
-      assert_same(e, $exc)
-    end;
-  end
-
-  def test_wrong_backtrace
-    assert_separately([], "#{<<-"begin;"}\n#{<<-"end;"}")
-    begin;
-      class Exception
-        undef backtrace
-        def backtrace(a)
-        end
-      end
-
-      assert_raise(RuntimeError) {
-        raise RuntimeError, "hello"
-      }
-    end;
+    end
   end
 end
