@@ -1,3 +1,4 @@
+# frozen_string_literal: false
 require 'test/unit'
 
 class TestMath < Test::Unit::TestCase
@@ -17,6 +18,12 @@ class TestMath < Test::Unit::TestCase
   end
   alias check assert_float
 
+  def assert_float_and_int(exp_ary, act_ary)
+    flo_exp, int_exp, flo_act, int_act = *exp_ary, *act_ary
+    assert_float(flo_exp, flo_act)
+    assert_equal(int_exp, int_act)
+  end
+
   def test_atan2
     check(+0.0, Math.atan2(+0.0, +0.0))
     check(-0.0, Math.atan2(-0.0, +0.0))
@@ -42,6 +49,7 @@ class TestMath < Test::Unit::TestCase
     check(0.0,  Math.cos(2 * Math::PI / 4))
     check(-1.0, Math.cos(4 * Math::PI / 4))
     check(0.0,  Math.cos(6 * Math::PI / 4))
+    check(0.5403023058681398,  Math.cos(1))
   end
 
   def test_sin
@@ -143,7 +151,8 @@ class TestMath < Test::Unit::TestCase
     check(1, Math.log(10, 10))
     check(2, Math.log(100, 10))
     check(Math.log(2.0 ** 64), Math.log(1 << 64))
-    assert_equal(1.0/0, Math.log(1.0/0))
+    check(Math.log(2) * 1024.0, Math.log(2 ** 1024))
+    assert_nothing_raised { assert_infinity(Math.log(1.0/0)) }
     assert_nothing_raised { assert_infinity(-Math.log(+0.0)) }
     assert_nothing_raised { assert_infinity(-Math.log(-0.0)) }
     assert_raise(Math::DomainError) { Math.log(-1.0) }
@@ -157,7 +166,8 @@ class TestMath < Test::Unit::TestCase
     check(1, Math.log2(2))
     check(2, Math.log2(4))
     check(Math.log2(2.0 ** 64), Math.log2(1 << 64))
-    assert_equal(1.0/0, Math.log2(1.0/0))
+    check(1024.0, Math.log2(2 ** 1024))
+    assert_nothing_raised { assert_infinity(Math.log2(1.0/0)) }
     assert_nothing_raised { assert_infinity(-Math.log2(+0.0)) }
     assert_nothing_raised { assert_infinity(-Math.log2(-0.0)) }
     assert_raise(Math::DomainError) { Math.log2(-1.0) }
@@ -168,7 +178,8 @@ class TestMath < Test::Unit::TestCase
     check(1, Math.log10(10))
     check(2, Math.log10(100))
     check(Math.log10(2.0 ** 64), Math.log10(1 << 64))
-    assert_equal(1.0/0, Math.log10(1.0/0))
+    check(Math.log10(2) * 1024, Math.log10(2 ** 1024))
+    assert_nothing_raised { assert_infinity(Math.log10(1.0/0)) }
     assert_nothing_raised { assert_infinity(-Math.log10(+0.0)) }
     assert_nothing_raised { assert_infinity(-Math.log10(-0.0)) }
     assert_raise(Math::DomainError) { Math.log10(-1.0) }
@@ -178,22 +189,25 @@ class TestMath < Test::Unit::TestCase
     check(0, Math.sqrt(0))
     check(1, Math.sqrt(1))
     check(2, Math.sqrt(4))
-    assert_equal(1.0/0, Math.sqrt(1.0/0))
+    assert_nothing_raised { assert_infinity(Math.sqrt(1.0/0)) }
     assert_equal("0.0", Math.sqrt(-0.0).to_s) # insure it is +0.0, not -0.0
     assert_raise(Math::DomainError) { Math.sqrt(-1.0) }
   end
 
+  def test_cbrt
+    check(1, Math.cbrt(1))
+    check(-2, Math.cbrt(-8))
+    check(3, Math.cbrt(27))
+    check(-0.1, Math.cbrt(-0.001))
+    assert_nothing_raised { assert_infinity(Math.cbrt(1.0/0)) }
+  end
+
   def test_frexp
-    check(0.0, Math.frexp(0.0).first)
-    assert_equal(0, Math.frexp(0).last)
-    check(0.5, Math.frexp(0.5).first)
-    assert_equal(0, Math.frexp(0.5).last)
-    check(0.5, Math.frexp(1.0).first)
-    assert_equal(1, Math.frexp(1.0).last)
-    check(0.5, Math.frexp(2.0).first)
-    assert_equal(2, Math.frexp(2.0).last)
-    check(0.75, Math.frexp(3.0).first)
-    assert_equal(2, Math.frexp(3.0).last)
+    assert_float_and_int([0.0,  0], Math.frexp(0.0))
+    assert_float_and_int([0.5,  0], Math.frexp(0.5))
+    assert_float_and_int([0.5,  1], Math.frexp(1.0))
+    assert_float_and_int([0.5,  2], Math.frexp(2.0))
+    assert_float_and_int([0.75, 2], Math.frexp(3.0))
   end
 
   def test_ldexp
@@ -230,6 +244,8 @@ class TestMath < Test::Unit::TestCase
     check(2, Math.gamma(3))
     check(15 * sqrt_pi / 8, Math.gamma(3.5))
     check(6, Math.gamma(4))
+    check(1.1240007277776077e+21, Math.gamma(23))
+    check(2.5852016738885062e+22, Math.gamma(24))
 
     # no SEGV [ruby-core:25257]
     31.upto(65) do |i|
@@ -239,63 +255,77 @@ class TestMath < Test::Unit::TestCase
     end
 
     assert_raise(Math::DomainError) { Math.gamma(-Float::INFINITY) }
-    x = Math.gamma(-0.0)
-    mesg = "Math.gamma(-0.0) should be -INF"
-    assert_infinity(x, mesg)
-    assert_operator(x, :<, 0, mesg)
   end
 
   def test_lgamma
     sqrt_pi = Math.sqrt(Math::PI)
-
-    g, s = Math.lgamma(-1.5)
-    check(Math.log(4 * sqrt_pi / 3), g)
-    assert_equal(s, 1)
-
-    g, s = Math.lgamma(-0.5)
-    check(Math.log(2 * sqrt_pi), g)
-    assert_equal(s, -1)
-
-    g, s = Math.lgamma(0.5)
-    check(Math.log(sqrt_pi), g)
-    assert_equal(s, 1)
-
-    assert_equal([0, 1], Math.lgamma(1))
-
-    g, s = Math.lgamma(1.5)
-    check(Math.log(sqrt_pi / 2), g)
-    assert_equal(s, 1)
-
-    assert_equal([0, 1], Math.lgamma(2))
-
-    g, s = Math.lgamma(2.5)
-    check(Math.log(3 * sqrt_pi / 4), g)
-    assert_equal(s, 1)
-
-    g, s = Math.lgamma(3)
-    check(Math.log(2), g)
-    assert_equal(s, 1)
-
-    g, s = Math.lgamma(3.5)
-    check(Math.log(15 * sqrt_pi / 8), g)
-    assert_equal(s, 1)
-
-    g, s = Math.lgamma(4)
-    check(Math.log(6), g)
-    assert_equal(s, 1)
+    assert_float_and_int([Math.log(4 * sqrt_pi / 3),  1], Math.lgamma(-1.5))
+    assert_float_and_int([Math.log(2 * sqrt_pi),     -1], Math.lgamma(-0.5))
+    assert_float_and_int([Math.log(sqrt_pi),          1], Math.lgamma(0.5))
+    assert_float_and_int([0,                          1], Math.lgamma(1))
+    assert_float_and_int([Math.log(sqrt_pi / 2),      1], Math.lgamma(1.5))
+    assert_float_and_int([0,                          1], Math.lgamma(2))
+    assert_float_and_int([Math.log(3 * sqrt_pi / 4),  1], Math.lgamma(2.5))
+    assert_float_and_int([Math.log(2),                1], Math.lgamma(3))
+    assert_float_and_int([Math.log(15 * sqrt_pi / 8), 1], Math.lgamma(3.5))
+    assert_float_and_int([Math.log(6),                1], Math.lgamma(4))
 
     assert_raise(Math::DomainError) { Math.lgamma(-Float::INFINITY) }
-    x, sign = Math.lgamma(-0.0)
-    mesg = "Math.lgamma(-0.0) should be [INF, -1]"
-    assert_infinity(x, mesg)
-    assert_operator(x, :>, 0, mesg)
-    assert_equal(-1, sign, mesg)
   end
 
-  def test_cbrt
-    check(1, Math.cbrt(1))
-    check(-2, Math.cbrt(-8))
-    check(3, Math.cbrt(27))
-    check(-0.1, Math.cbrt(-0.001))
+  def test_fixnum_to_f
+    check(12.0, Math.sqrt(144))
+  end
+
+  def test_override_fixnum_to_f
+    Fixnum.class_eval do
+      alias _to_f to_f
+      def to_f
+        (self + 1)._to_f
+      end
+    end
+
+    check(Math.cos((0 + 1)._to_f), Math.cos(0))
+    check(Math.exp((0 + 1)._to_f), Math.exp(0))
+    check(Math.log((0 + 1)._to_f), Math.log(0))
+
+    Fixnum.class_eval { alias to_f _to_f }
+  end
+
+  def test_bignum_to_f
+    check((1 << 65).to_f, Math.sqrt(1 << 130))
+  end
+
+  def test_override_bignum_to_f
+    Bignum.class_eval do
+      alias _to_f to_f
+      def to_f
+        (self << 1)._to_f
+      end
+    end
+
+    check(Math.cos((1 << 62 << 1)._to_f),  Math.cos(1 << 62))
+    check(Math.log((1 << 62 << 1)._to_f),  Math.log(1 << 62))
+
+    Bignum.class_eval { alias to_f _to_f }
+  end
+
+  def test_rational_to_f
+    check((2 ** 31).fdiv(3 ** 20), Math.sqrt((2 ** 62)/(3 ** 40).to_r))
+  end
+
+  def test_override_rational_to_f
+    Rational.class_eval do
+      alias _to_f to_f
+      def to_f
+        (self + 1)._to_f
+      end
+    end
+
+    check(Math.cos((0r + 1)._to_f), Math.cos(0r))
+    check(Math.exp((0r + 1)._to_f), Math.exp(0r))
+    check(Math.log((0r + 1)._to_f), Math.log(0r))
+
+    Rational.class_eval { alias to_f _to_f }
   end
 end
