@@ -1,4 +1,3 @@
-# coding: utf-8
 # frozen_string_literal: false
 require 'test/unit'
 require 'tempfile'
@@ -181,11 +180,7 @@ class TestProcess < Test::Unit::TestCase
     io.close
 
     assert_raise(ArgumentError) { system(*TRUECOMMAND, :pgroup=>-1) }
-    IO.popen([RUBY, '-egets'], 'w') do |f|
-      assert_raise(Errno::EPERM) {
-        Process.wait spawn(*TRUECOMMAND, :pgroup=>f.pid)
-      }
-    end
+    assert_raise(Errno::EPERM) { Process.wait spawn(*TRUECOMMAND, :pgroup=>2) }
 
     io1 = IO.popen([RUBY, "-e", "print Process.getpgrp", :pgroup=>true])
     io2 = IO.popen([RUBY, "-e", "print Process.getpgrp", :pgroup=>io1.pid])
@@ -430,16 +425,6 @@ class TestProcess < Test::Unit::TestCase
     }
   end
 
-  def test_execopts_open_chdir_m17n_path
-    with_tmpchdir {|d|
-      Dir.mkdir "テスト"
-      system(*PWD, :chdir => "テスト", :out => "open_chdir_テスト")
-      assert_file.exist?("open_chdir_テスト")
-      assert_file.not_exist?("テスト/open_chdir_テスト")
-      assert_equal("#{d}/テスト", File.read("open_chdir_テスト").chomp.encode(__ENCODING__))
-    }
-  end if windows? || Encoding.find('locale') == Encoding::UTF_8
-
   def test_execopts_open_failure
     with_tmpchdir {|d|
       assert_raise_with_message(Errno::ENOENT, %r"d/notexist") {
@@ -1817,15 +1802,7 @@ class TestProcess < Test::Unit::TestCase
       assert_nothing_raised(feature6975) do
         begin
           g = IO.popen([RUBY, "-e", "print Process.gid", gid: group], &:read)
-          # AIX allows a non-root process to setgid to its supplementary group,
-          # while other UNIXes do not. (This might be AIX's violation of the POSIX standard.)
-          # However, Ruby does not allow a setgid'ed Ruby process to use the -e option.
-          # As a result, the Ruby process invoked by "IO.popen([RUBY, "-e", ..." above fails
-          # with a message like "no -e allowed while running setgid (SecurityError)" to stderr,
-          # the exis status is set to 1, and the variable "g" is set to an empty string.
-          # To conclude, on AIX, if the "gid" variable is a supplementary group,
-          # the assert_equal next can fail, so skip it.
-          assert_equal(gid, g, feature6975) unless $?.exitstatus == 1 && /aix/ =~ RUBY_PLATFORM && gid != Process.gid
+          assert_equal(gid, g, feature6975)
         rescue Errno::EPERM, NotImplementedError
         end
       end
@@ -2260,22 +2237,4 @@ EOS
       system(bin, "--disable=gems", "-w", "-e", "puts ARGV", *args)
     end;
   end
-
-  def test_to_hash_on_arguments
-    all_assertions do |a|
-      %w[Array String].each do |type|
-        a.for(type) do
-          assert_separately(['-', EnvUtil.rubybin], <<~"END;")
-          class #{type}
-            def to_hash
-              raise "[Bug-12355]: #{type}#to_hash is called"
-            end
-          end
-          ex = ARGV[0]
-          assert_equal(true, system([ex, ex], "-e", ""))
-          END;
-        end
-      end
-    end
-  end
 end
