@@ -1,3 +1,4 @@
+# frozen_string_literal: false
 require 'test/unit'
 
 class TestSetTraceFunc < Test::Unit::TestCase
@@ -464,7 +465,7 @@ class TestSetTraceFunc < Test::Unit::TestCase
     EOF
     self.class.class_eval{remove_const(:XYZZY)}
     ensure
-      trace.disable if trace && trace.enabled?
+      trace.disable if trace&.enabled?
     end
 
     answer_events = [
@@ -1040,7 +1041,7 @@ class TestSetTraceFunc < Test::Unit::TestCase
 
   def test_isolated_raise_in_trace
     bug9088 = '[ruby-dev:47793] [Bug #9088]'
-    assert_ruby_status([], <<-END, bug9088)
+    assert_in_out_err([], <<-END, [], [], bug9088)
     set_trace_func proc {raise rescue nil}
     1.times {break}
     END
@@ -1280,16 +1281,13 @@ class TestSetTraceFunc < Test::Unit::TestCase
   end
 
   def test_recursive
-    assert_ruby_status [], %q{
-      stack = []
+    assert_in_out_err([], %q{\
       TracePoint.new(:c_call){|tp|
-        p 2
-        stack << tp.method_id
+        p tp.method_id
       }.enable{
         p 1
       }
-      raise if stack != [:p, :hash, :inspect]
-    }, '[Bug #9940]'
+    }, %w[:p :inspect 1], [], '[Bug #9940]')
   end
 
   def method_prefix event
@@ -1401,25 +1399,6 @@ class TestSetTraceFunc < Test::Unit::TestCase
     assert_equal [__LINE__ - 3, __LINE__ - 2], lines, 'Bug #10449'
   end
 
-  def test_elsif_line_event
-    bug10763 = '[ruby-core:67720] [Bug #10763]'
-    lines = []
-    line = nil
-
-    TracePoint.new(:line){|tp|
-      next unless target_thread?
-      lines << tp.lineno if line
-    }.enable{
-      line = __LINE__
-      if !line
-        1
-      elsif line
-        2
-      end
-    }
-    assert_equal [line+1, line+3, line+4], lines, bug10763
-  end
-
   class Bug10724
     def initialize
       loop{return}
@@ -1439,4 +1418,55 @@ class TestSetTraceFunc < Test::Unit::TestCase
 
     assert_equal([:call, :return], evs)
   end
+
+  require 'fiber'
+  def test_fiber_switch
+    # test for resume/yield
+    evs = []
+    TracePoint.new(:fiber_switch){|tp|
+      next unless target_thread?
+      evs << tp.event
+    }.enable{
+      f = Fiber.new{
+        Fiber.yield
+        Fiber.yield
+        Fiber.yield
+      }
+      f.resume
+      f.resume
+      f.resume
+      f.resume
+      begin
+        f.resume
+      rescue FiberError
+      end
+    }
+    assert_equal 8, evs.size
+    evs.each{|ev|
+      assert_equal ev, :fiber_switch
+    }
+
+    # test for transfer
+    evs = []
+    TracePoint.new(:fiber_switch){|tp|
+      next unless target_thread?
+      evs << tp.event
+    }.enable{
+      f1 = f2 = nil
+      f1 = Fiber.new{
+        f2.transfer
+        f2.transfer
+        Fiber.yield :ok
+      }
+      f2 = Fiber.new{
+        f1.transfer
+        f1.transfer
+      }
+      assert_equal :ok, f1.resume
+    }
+    assert_equal 6, evs.size
+    evs.each{|ev|
+      assert_equal ev, :fiber_switch
+    }
+  end
 end
