@@ -149,11 +149,6 @@ class TestSprintf < Test::Unit::TestCase
     assert_equal(" Inf", sprintf("% e", inf), '[ruby-dev:34002]')
   end
 
-  def test_bignum
-    assert_match(/\A10{120}\.0+\z/, sprintf("%f", 100**60))
-    assert_match(/\A10{180}\.0+\z/, sprintf("%f", 1000**60))
-  end
-
   def test_rational
     assert_match(/\A0\.10+\z/, sprintf("%.60f", 0.1r))
     assert_match(/\A0\.010+\z/, sprintf("%.60f", 0.01r))
@@ -177,10 +172,6 @@ class TestSprintf < Test::Unit::TestCase
     assert_equal("x"*10+"     1.0", sprintf("x"*10+"%8.1f", 1r))
   end
 
-  def test_rational_precision
-    assert_match(/\A0\.\d{600}\z/, sprintf("%.600f", 600**~60))
-  end
-
   def test_hash
     options = {:capture=>/\d+/}
     assert_equal("with options {:capture=>/\\d+/}", sprintf("with options %p" % options))
