@@ -157,15 +157,6 @@ class TestSymbol < Test::Unit::TestCase
     assert_equal(1, first, bug11594)
   end
 
-  private def return_from_proc
-    Proc.new { return 1 }.tap(&:call)
-  end
-
-  def test_return_from_symbol_proc
-    bug12462 = '[ruby-core:75856] [Bug #12462]'
-    assert_equal(1, return_from_proc, bug12462)
-  end
-
   def test_to_proc_for_hash_each
     bug11830 = '[ruby-core:72205] [Bug #11830]'
     assert_normal_exit(<<-'end;', bug11830) # do
@@ -185,15 +176,6 @@ class TestSymbol < Test::Unit::TestCase
     end;
   end
 
-  def test_to_proc_binding
-    assert_separately([], <<~"end;", timeout: 1) # do
-      bug12137 = '[ruby-core:74100] [Bug #12137]'
-      assert_raise(ArgumentError, bug12137) {
-        :succ.to_proc.binding
-      }
-    end;
-  end
-
   def test_call
     o = Object.new
     def o.foo(x, y); x + y; end
@@ -229,35 +211,6 @@ class TestSymbol < Test::Unit::TestCase
     assert_equal([false, false], m2.call(self, m2), "#{bug8531} nested without block")
   end
 
-  def test_block_curry_proc
-    assert_separately([], "#{<<-"begin;"}\n#{<<-"end;"}")
-    begin;
-    b = proc { true }.curry
-    assert(b.call, "without block")
-    assert(b.call { |o| o.to_s }, "with block")
-    assert(b.call(&:to_s), "with sym block")
-    end;
-  end
-
-  def test_block_curry_lambda
-    assert_separately([], "#{<<-"begin;"}\n#{<<-"end;"}")
-    begin;
-    b = lambda { true }.curry
-    assert(b.call, "without block")
-    assert(b.call { |o| o.to_s }, "with block")
-    assert(b.call(&:to_s), "with sym block")
-    end;
-  end
-
-  def test_block_method_to_proc
-    assert_separately([], "#{<<-"begin;"}\n#{<<-"end;"}")
-    begin;
-    b = method(:tap).to_proc
-    assert(b.call { |o| o.to_s }, "with block")
-    assert(b.call(&:to_s), "with sym block")
-    end;
-  end
-
   def test_succ
     assert_equal(:fop, :foo.succ)
   end
