@@ -1,12 +1,14 @@
-require File.join(File.expand_path(File.dirname(__FILE__)), 'gemutilities')
+# frozen_string_literal: false
+require 'rubygems/test_case'
 require 'rubygems/ext'
 
-class TestGemExtConfigureBuilder < RubyGemTestCase
+class TestGemExtConfigureBuilder < Gem::TestCase
 
   def setup
     super
 
-    @makefile_body =  "all:\n\t@echo ok\ninstall:\n\t@echo ok"
+    @makefile_body =
+      "clean:\n\t@echo ok\nall:\n\t@echo ok\ninstall:\n\t@echo ok"
 
     @ext = File.join @tempdir, 'ext'
     @dest_path = File.join @tempdir, 'prefix'
@@ -16,7 +18,7 @@ class TestGemExtConfigureBuilder < RubyGemTestCase
   end
 
   def test_self_build
-    return if RUBY_PLATFORM =~ /mswin/ # HACK
+    skip("test_self_build skipped on MS Windows (VC++)") if vc_windows?
 
     File.open File.join(@ext, './configure'), 'w' do |configure|
       configure.puts "#!/bin/sh\necho \"#{@makefile_body}\" > Makefile"
@@ -28,16 +30,22 @@ class TestGemExtConfigureBuilder < RubyGemTestCase
       Gem::Ext::ConfigureBuilder.build nil, nil, @dest_path, output
     end
 
+    assert_match(/^current directory:/, output.shift)
     assert_equal "sh ./configure --prefix=#{@dest_path}", output.shift
     assert_equal "", output.shift
-    assert_equal "make", output.shift
+    assert_match(/^current directory:/, output.shift)
+    assert_contains_make_command 'clean', output.shift
     assert_match(/^ok$/m, output.shift)
-    assert_equal "make install", output.shift
+    assert_match(/^current directory:/, output.shift)
+    assert_contains_make_command '', output.shift
+    assert_match(/^ok$/m, output.shift)
+    assert_match(/^current directory:/, output.shift)
+    assert_contains_make_command 'install', output.shift
     assert_match(/^ok$/m, output.shift)
   end
 
   def test_self_build_fail
-    return if RUBY_PLATFORM =~ /mswin/ # HACK
+    skip("test_self_build_fail skipped on MS Windows (VC++)") if vc_windows?
     output = []
 
     error = assert_raises Gem::InstallError do
@@ -46,23 +54,22 @@ class TestGemExtConfigureBuilder < RubyGemTestCase
       end
     end
 
-    shell_error_msg = %r{(\./configure: .*)|(Can't open \./configure(?:: No such file or directory)?)}
+    shell_error_msg = %r{(\./configure: .*)|((?:Can't|cannot) open \./configure(?:: No such file or directory)?)}
     sh_prefix_configure = "sh ./configure --prefix="
 
-    expected = %r(configure failed:
-
-#{Regexp.escape sh_prefix_configure}#{Regexp.escape @dest_path}
-.*?: #{shell_error_msg}
-)
-
-    assert_match expected, error.message
+    assert_match 'configure failed', error.message
 
+    assert_match(/^current directory:/, output.shift)
     assert_equal "#{sh_prefix_configure}#{@dest_path}", output.shift
     assert_match %r(#{shell_error_msg}), output.shift
     assert_equal true, output.empty?
   end
 
   def test_self_build_has_makefile
+    if vc_windows? && !nmake_found?
+      skip("test_self_build_has_makefile skipped - nmake not found")
+    end
+
     File.open File.join(@ext, 'Makefile'), 'w' do |makefile|
       makefile.puts @makefile_body
     end
@@ -72,15 +79,9 @@ class TestGemExtConfigureBuilder < RubyGemTestCase
       Gem::Ext::ConfigureBuilder.build nil, nil, @dest_path, output
     end
 
-    case RUBY_PLATFORM
-    when /mswin/ then
-      assert_equal 'nmake', output[0]
-      assert_equal 'nmake install', output[2]
-    else
-      assert_equal 'make', output[0]
-      assert_equal 'make install', output[2]
-    end
+    assert_contains_make_command 'clean', output[1]
+    assert_contains_make_command '', output[4]
+    assert_contains_make_command 'install', output[7]
   end
 
 end
-
