@@ -1,4 +1,5 @@
 # coding: UTF-8
+# frozen_string_literal: false
 
 require 'rubygems/test_case'
 require 'rubygems/ext'
@@ -33,55 +34,58 @@ class TestGemExtExtConfBuilder < Gem::TestCase
       assert_same result, output
     end
 
-    assert_match(/^#{Gem.ruby} extconf.rb/, output[0])
-    assert_equal "creating Makefile\n", output[1]
-    assert_contains_make_command '', output[2]
-    assert_contains_make_command 'install', output[4]
+    assert_match(/^current directory:/, output[0])
+    assert_match(/^#{Gem.ruby}.* extconf.rb/, output[1])
+    assert_equal "creating Makefile\n", output[2]
+    assert_match(/^current directory:/, output[3])
+    assert_contains_make_command 'clean', output[4]
+    assert_contains_make_command '', output[7]
+    assert_contains_make_command 'install', output[10]
     assert_empty Dir.glob(File.join(@ext, 'siteconf*.rb'))
   end
 
   def test_class_build_rbconfig_make_prog
-    configure_args = RbConfig::CONFIG['configure_args']
+    configure_args do
 
-    File.open File.join(@ext, 'extconf.rb'), 'w' do |extconf|
-      extconf.puts "require 'mkmf'\ncreate_makefile 'foo'"
-    end
+      File.open File.join(@ext, 'extconf.rb'), 'w' do |extconf|
+        extconf.puts "require 'mkmf'\ncreate_makefile 'foo'"
+      end
 
-    output = []
+      output = []
 
-    Dir.chdir @ext do
-      Gem::Ext::ExtConfBuilder.build 'extconf.rb', nil, @dest_path, output
-    end
-
-    assert_equal "creating Makefile\n", output[1]
-    assert_contains_make_command '', output[2]
-    assert_contains_make_command 'install', output[4]
-  ensure
-    RbConfig::CONFIG['configure_args'] = configure_args
-  end
-
-  def test_class_build_env_make
-    configure_args, env_make = RbConfig::CONFIG['configure_args'], ENV.delete('make')
-    RbConfig::CONFIG['configure_args'] = ''
-    ENV['make'] = 'anothermake'
-
-    File.open File.join(@ext, 'extconf.rb'), 'w' do |extconf|
-      extconf.puts "require 'mkmf'\ncreate_makefile 'foo'"
-    end
-
-    output = []
-
-    assert_raises Gem::InstallError do
       Dir.chdir @ext do
         Gem::Ext::ExtConfBuilder.build 'extconf.rb', nil, @dest_path, output
       end
-    end
 
-    assert_equal "creating Makefile\n", output[1]
-    assert_contains_make_command '', output[2]
+      assert_equal "creating Makefile\n", output[2]
+      assert_contains_make_command 'clean', output[4]
+      assert_contains_make_command '', output[7]
+      assert_contains_make_command 'install', output[10]
+    end
+  end
+
+  def test_class_build_env_make
+    env_make = ENV.delete 'MAKE'
+    ENV['MAKE'] = 'anothermake'
+
+    configure_args '' do
+      File.open File.join(@ext, 'extconf.rb'), 'w' do |extconf|
+        extconf.puts "require 'mkmf'\ncreate_makefile 'foo'"
+      end
+
+      output = []
+
+      assert_raises Gem::InstallError do
+        Dir.chdir @ext do
+          Gem::Ext::ExtConfBuilder.build 'extconf.rb', nil, @dest_path, output
+        end
+      end
+
+      assert_equal "creating Makefile\n",   output[2]
+      assert_contains_make_command 'clean', output[4]
+    end
   ensure
-    RbConfig::CONFIG['configure_args'] = configure_args
-    ENV['make'] = env_make
+    ENV['MAKE'] = env_make
   end
 
   def test_class_build_extconf_fail
@@ -103,12 +107,12 @@ class TestGemExtExtConfBuilder < Gem::TestCase
       end
     end
 
-    assert_match(/\Aextconf failed:
+    assert_equal 'extconf failed, exit code 1', error.message
 
-#{Gem.ruby} extconf.rb.*
-checking for main\(\) in .*?nonexistent/m, error.message)
+    assert_match(/^#{Gem.ruby}.* extconf.rb/, output[1])
+    assert_match(File.join(@dest_path, 'mkmf.log'), output[4])
 
-    assert_equal("#{Gem.ruby} extconf.rb", output[0])
+    assert_path_exists File.join @dest_path, 'mkmf.log'
   end
 
   def test_class_build_unconventional
@@ -120,11 +124,17 @@ checking for main\(\) in .*?nonexistent/m, error.message)
       extconf.puts <<-'EXTCONF'
 include RbConfig
 
-ruby_exe = "#{CONFIG['RUBY_INSTALL_NAME']}#{CONFIG['EXEEXT']}"
-ruby = File.join CONFIG['bindir'], ruby_exe
+ruby =
+  if ENV['RUBY'] then
+    ENV['RUBY']
+  else
+    ruby_exe = "#{CONFIG['RUBY_INSTALL_NAME']}#{CONFIG['EXEEXT']}"
+    File.join CONFIG['bindir'], ruby_exe
+  end
 
 open 'Makefile', 'w' do |io|
   io.write <<-Makefile
+clean: ruby
 all: ruby
 install: ruby
 
@@ -142,8 +152,9 @@ end
       Gem::Ext::ExtConfBuilder.build 'extconf.rb', nil, @dest_path, output
     end
 
-    assert_contains_make_command '', output[2]
-    assert_contains_make_command 'install', output[4]
+    assert_contains_make_command 'clean', output[4]
+    assert_contains_make_command '', output[7]
+    assert_contains_make_command 'install', output[10]
     assert_empty Dir.glob(File.join(@ext, 'siteconf*.rb'))
   end
 
@@ -158,6 +169,7 @@ end
       makefile.puts "# π"
       makefile.puts "RUBYARCHDIR = $(foo)$(target_prefix)"
       makefile.puts "RUBYLIBDIR = $(bar)$(target_prefix)"
+      makefile.puts "clean:"
       makefile.puts "all:"
       makefile.puts "install:"
     end
@@ -166,8 +178,9 @@ end
       Gem::Ext::ExtConfBuilder.make @ext, output
     end
 
-    assert_contains_make_command '', output[0]
-    assert_contains_make_command 'install', output[2]
+    assert_contains_make_command 'clean', output[1]
+    assert_contains_make_command '', output[4]
+    assert_contains_make_command 'install', output[7]
   end
 
   def test_class_make_no_Makefile
@@ -177,13 +190,21 @@ end
       end
     end
 
-    expected = <<-EOF.strip
-Makefile not found:
+    assert_equal 'Makefile not found', error.message
+  end
 
-output
-    EOF
+  def configure_args args = nil
+    configure_args = RbConfig::CONFIG['configure_args']
+    RbConfig::CONFIG['configure_args'] = args if args
 
-    assert_equal expected, error.message
+    yield
+
+  ensure
+    if configure_args then
+      RbConfig::CONFIG['configure_args'] = configure_args
+    else
+      RbConfig::CONFIG.delete 'configure_args'
+    end
   end
 
 end
