@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'test/unit'
 require 'socket'
 
@@ -23,19 +25,6 @@ class TestSocketOption < Test::Unit::TestCase
     assert_equal(true, opt.bool)
     opt = Socket::Option.int(:INET, :SOCKET, :KEEPALIVE, 2)
     assert_equal(true, opt.bool)
-    begin
-      Socket.open(:INET, :STREAM) {|s|
-        s.setsockopt(Socket::Option.bool(:INET, :SOCKET, :KEEPALIVE, true))
-        assert_equal(true, s.getsockopt(Socket::SOL_SOCKET, Socket::SO_KEEPALIVE).bool)
-        s.setsockopt(Socket::Option.bool(:INET, :SOCKET, :KEEPALIVE, false))
-        assert_equal(false, s.getsockopt(Socket::SOL_SOCKET, Socket::SO_KEEPALIVE).bool)
-      }
-    rescue TypeError
-      if /aix/ =~ RUBY_PLATFORM
-        skip "Known bug in getsockopt(2) on AIX"
-      end
-      raise $!
-    end
   end
 
   def test_ipv4_multicast_loop
