@@ -2,7 +2,7 @@
 
   variable.c -
 
-  $Author: naruse $
+  $Author$
   created at: Tue Apr 19 23:55:15 JST 1994
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -2445,9 +2445,6 @@ rb_const_list(void *data)
  *  modules (example at start of section), unless the <i>inherit</i>
  *  parameter is set to <code>false</code>.
  *
- *  The implementation makes no guarantees about the order in which the
- *  constants are yielded.
- *
  *    IO.constants.include?(:SYNC)        #=> true
  *    IO.constants(false).include?(:SYNC) #=> false
  *
