@@ -1,10 +1,14 @@
+#if defined(__MINGW32__)
+/* before stdio.h in ruby/define.h */
+# define MINGW_HAS_SECURE_API 1
+#endif
 #include "ruby/ruby.h"
 #include "ruby/encoding.h"
-#include "ruby/thread.h"
 #include "internal.h"
 #include <winbase.h>
 #include <wchar.h>
 #include <shlwapi.h>
+#include "win32/file.h"
 
 #ifndef INVALID_FILE_ATTRIBUTES
 # define INVALID_FILE_ATTRIBUTES ((DWORD)-1)
@@ -161,8 +165,11 @@ code_page_i(st_data_t name, st_data_t idx, st_data_t arg)
 	    USHORT *table = cp->table;
 	    if (count <= idx) {
 		unsigned int i = count;
-		cp->count = count = (((idx + 4) & ~31) | 28);
-		cp->table = table = realloc(table, count * sizeof(*table));
+		count = (((idx + 4) & ~31) | 28);
+		table = realloc(table, count * sizeof(*table));
+		if (!table) return ST_CONTINUE;
+		cp->count = count;
+		cp->table = table;
 		while (i < count) table[i++] = INVALID_CODE_PAGE;
 	    }
 	    table[idx] = (USHORT)code_page;
@@ -348,7 +355,14 @@ rb_file_expand_path_internal(VALUE fname, VALUE dname, int abs_mode, int long_na
 
     /* convert char * to wchar_t */
     if (!NIL_P(path)) {
-	wpath = mbstr_to_wstr(cp, RSTRING_PTR(path), (int)RSTRING_LEN(path), &wpath_len);
+	const long path_len = RSTRING_LEN(path);
+#if SIZEOF_INT < SIZEOF_LONG
+	if ((long)(int)path_len != path_len) {
+	    rb_raise(rb_eRangeError, "path (%ld bytes) is too long",
+		     path_len);
+	}
+#endif
+	wpath = mbstr_to_wstr(cp, RSTRING_PTR(path), path_len, &wpath_len);
 	wpath_pos = wpath;
     }
 
@@ -423,7 +437,15 @@ rb_file_expand_path_internal(VALUE fname, VALUE dname, int abs_mode, int long_na
 
 	/* convert char * to wchar_t */
 	if (!NIL_P(dir)) {
-	    wdir = mbstr_to_wstr(cp, RSTRING_PTR(dir), (int)RSTRING_LEN(dir), &wdir_len);
+	    const long dir_len = RSTRING_LEN(dir);
+#if SIZEOF_INT < SIZEOF_LONG
+	    if ((long)(int)dir_len != dir_len) {
+		if (wpath) xfree(wpath);
+		rb_raise(rb_eRangeError, "base directory (%ld bytes) is too long",
+			 dir_len);
+	    }
+#endif
+	    wdir = mbstr_to_wstr(cp, RSTRING_PTR(dir), dir_len, &wdir_len);
 	    wdir_pos = wdir;
 	}
 
@@ -626,7 +648,7 @@ rb_file_expand_path_internal(VALUE fname, VALUE dname, int abs_mode, int long_na
     if (whome)
 	xfree(whome);
 
-    if (wfullpath && wfullpath != wfullpath_buffer)
+    if (wfullpath != wfullpath_buffer)
 	xfree(wfullpath);
 
     if (fullpath)
@@ -636,12 +658,45 @@ rb_file_expand_path_internal(VALUE fname, VALUE dname, int abs_mode, int long_na
     return result;
 }
 
-static void *
-loadopen_func(void *wpath)
+VALUE
+rb_readlink(VALUE path, rb_encoding *resultenc)
 {
-    return (void *)CreateFileW(wpath, GENERIC_READ,
-			       FILE_SHARE_READ | FILE_SHARE_WRITE,
-			       NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
+    DWORD len;
+    VALUE wtmp = 0, wpathbuf, str;
+    rb_w32_reparse_buffer_t rbuf, *rp = &rbuf;
+    WCHAR *wpath, *wbuf;
+    rb_encoding *enc;
+    UINT cp, path_cp;
+    int e;
+
+    FilePathValue(path);
+    enc = rb_enc_get(path);
+    cp = path_cp = code_page(enc);
+    if (cp == INVALID_CODE_PAGE) {
+	path = fix_string_encoding(path, enc);
+	cp = CP_UTF8;
+    }
+    len = MultiByteToWideChar(cp, 0, RSTRING_PTR(path), RSTRING_LEN(path), NULL, 0);
+    wpath = ALLOCV_N(WCHAR, wpathbuf, len+1);
+    MultiByteToWideChar(cp, 0, RSTRING_PTR(path), RSTRING_LEN(path), wpath, len);
+    wpath[len] = L'\0';
+    e = rb_w32_read_reparse_point(wpath, rp, sizeof(rbuf), &wbuf, &len);
+    if (e == ERROR_MORE_DATA) {
+	size_t size = rb_w32_reparse_buffer_size(len + 1);
+	rp = ALLOCV(wtmp, size);
+	e = rb_w32_read_reparse_point(wpath, rp, size, &wbuf, &len);
+    }
+    ALLOCV_END(wpathbuf);
+    if (e) {
+	ALLOCV_END(wtmp);
+	rb_syserr_fail_path(rb_w32_map_errno(e), path);
+    }
+    enc = resultenc;
+    cp = path_cp = code_page(enc);
+    if (cp == INVALID_CODE_PAGE) cp = CP_UTF8;
+    str = append_wstr(rb_enc_str_new(0, 0, enc), wbuf, len, cp, path_cp, enc);
+    ALLOCV_END(wtmp);
+    return str;
 }
 
 int
@@ -661,8 +716,9 @@ rb_file_load_ok(const char *path)
 	ret = 0;
     }
     else {
-	HANDLE h = (HANDLE)rb_thread_call_without_gvl(loadopen_func, (void *)wpath,
-						      RUBY_UBF_IO, 0);
+	HANDLE h = CreateFileW(wpath, GENERIC_READ,
+			       FILE_SHARE_READ | FILE_SHARE_WRITE,
+			       NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
 	if (h != INVALID_HANDLE_VALUE) {
 	    CloseHandle(h);
 	}
@@ -674,6 +730,35 @@ rb_file_load_ok(const char *path)
     return ret;
 }
 
+int
+rb_freopen(VALUE fname, const char *mode, FILE *file)
+{
+    WCHAR *wname, wmode[4];
+    VALUE wtmp;
+    char *name;
+    long len;
+    int e = 0, n = MultiByteToWideChar(CP_ACP, 0, mode, -1, NULL, 0);
+    if (n > numberof(wmode)) return EINVAL;
+    MultiByteToWideChar(CP_ACP, 0, mode, -1, wmode, numberof(wmode));
+    RSTRING_GETMEM(fname, name, len);
+    n = rb_long2int(len);
+    len = MultiByteToWideChar(CP_UTF8, 0, name, n, NULL, 0);
+    wname = ALLOCV_N(WCHAR, wtmp, len + 1);
+    len = MultiByteToWideChar(CP_UTF8, 0, name, n, wname, len);
+    wname[len] = L'\0';
+    RB_GC_GUARD(fname);
+#if RUBY_MSVCRT_VERSION < 80 && !defined(HAVE__WFREOPEN_S)
+    e = _wfreopen(wname, wmode, file) ? 0 : errno;
+#else
+    {
+	FILE *newfp = 0;
+	e = _wfreopen_s(&newfp, wname, wmode, file);
+    }
+#endif
+    ALLOCV_END(wtmp);
+    return e;
+}
+
 void
 Init_w32_codepage(void)
 {
